@@ -1,5 +1,5 @@
 /*
-    $Id: vcddump.c,v 1.8 2003/01/30 16:55:58 rocky Exp $
+    $Id: vcddump.c,v 1.19 2003/03/16 01:05:11 rocky Exp $
 
     Copyright (C) 2001,2002 Herbert Valerio Riedel <hvr@gnu.org>
     Copyright (C) 2002,2003 Rocky Bernstein <rocky@panix.com>
@@ -33,6 +33,10 @@
 #include <stddef.h>
 
 #include <popt.h>
+/* Accomodate to older popt that doesn't support the "optional" flag */
+#ifndef POPT_ARGFLAG_OPTIONAL
+#define POPT_ARGFLAG_OPTIONAL 0
+#endif
 
 #include <libvcd/vcd_assert.h>
 #include <libvcd/vcd_bitvec.h>
@@ -48,8 +52,9 @@
 
 /* Eventually move above libvcd includes but having vcdinfo including. */
 #include <libvcd/vcdinfo.h>
+#include <libvcd/vcdinf.h>
 
-static const char _rcsid[] = "$Id: vcddump.c,v 1.8 2003/01/30 16:55:58 rocky Exp $";
+static const char _rcsid[] = "$Id: vcddump.c,v 1.19 2003/03/16 01:05:11 rocky Exp $";
 
 static const char DELIM[] = \
 "----------------------------------------" \
@@ -59,6 +64,7 @@ static const char DELIM[] = \
 static struct gl_t
 {
   vcdinfo_source_t source_type;
+  char *access_mode;
 
   /* Boolean values set by command-line options to reduce output.
      Note: because these are used by popt, the datatype here has to be 
@@ -295,7 +301,7 @@ dump_psd (const vcdinfo_obj_t *obj, bool ext)
             const PsdPlayListDescriptor *pld = (const void *) (psd + _rofs);
             
             int i;
-            uint16_t lid = vcdinfo_get_lid_from_pld(pld);
+            uint16_t lid = vcdinf_get_lid_from_pld(pld);
 
             fprintf (stdout,
                      " PSD[%.2d] (%s): play list descriptor\n"
@@ -305,18 +311,17 @@ dump_psd (const vcdinfo_obj_t *obj, bool ext)
                      n, vcdinfo_ofs2str (obj, ofs->offset, ext),
                      pld->noi, lid, 
                      _vcd_bool_str(vcdinfo_is_rejected(uint16_from_be(pld->lid))),
-                     vcdinfo_ofs2str(obj, vcdinfo_get_prev_from_pld(pld), ext),
-                     vcdinfo_ofs2str(obj, vcdinfo_get_next_from_pld(pld), ext),
-                     vcdinfo_ofs2str(obj, vcdinfo_get_return_from_pld(pld),
+                     vcdinfo_ofs2str(obj, vcdinf_get_prev_from_pld(pld), ext),
+                     vcdinfo_ofs2str(obj, vcdinf_get_next_from_pld(pld), ext),
+                     vcdinfo_ofs2str(obj, vcdinf_get_return_from_pld(pld),
                                      ext),
-                     vcdinfo_get_play_time(pld), vcdinfo_get_wait_time (pld),
+                     vcdinf_get_play_time(pld), vcdinfo_get_wait_time (pld),
                      vcdinfo_get_autowait_time(pld));
 
-            for (i = 0; i < pld->noi; i++)
-              {
-                fprintf (stdout, "  play-item[%d]: %s\n",
-                         i, vcdinfo_pin2str (uint16_from_be (pld->itemid[i])));
-              }
+            for (i = 0; i < pld->noi; i++) {
+              fprintf (stdout, "  play-item[%d]: %s\n", i, 
+                       vcdinfo_pin2str(vcdinf_get_play_item_from_pld(pld,i)));
+            }
             fprintf (stdout, "\n");
           }
           break;
@@ -340,6 +345,7 @@ dump_psd (const vcdinfo_obj_t *obj, bool ext)
             const PsdSelectionListDescriptor *d =
               (const void *) (psd + _rofs);
             int i;
+            const unsigned int lid=vcdinf_get_lid_from_psd(d);
 
             fprintf (stdout,
                      "  PSD[%.2d] (%s): %sselection list descriptor\n"
@@ -351,23 +357,24 @@ dump_psd (const vcdinfo_obj_t *obj, bool ext)
                      n, vcdinfo_ofs2str (obj, ofs->offset, ext),
                      (type == PSD_TYPE_EXT_SELECTION_LIST ? "extended " : ""),
                      *(uint8_t *) &d->flags,
-                     d->nos, 
-                     vcdinfo_get_bsn(d),
-                     vcdinfo_get_lid_from_psd(d),
-                     _vcd_bool_str (vcdinfo_get_lid_rejected_from_psd(d)),
-                     vcdinfo_ofs2str (obj, vcdinfo_get_prev_from_psd(d), ext),
-                     vcdinfo_ofs2str (obj, vcdinfo_get_next_from_psd(d), ext),
-                     vcdinfo_ofs2str (obj, vcdinfo_get_return_from_psd(d),ext),
-                     vcdinfo_ofs2str (obj, uint16_from_be (d->default_ofs), ext),
-                     vcdinfo_ofs2str (obj, vcdinfo_get_timeout_LID(d), ext),
+                     vcdinf_get_num_selections(d),
+                     vcdinf_get_bsn(d),
+                     lid, 
+                     _vcd_bool_str (vcdinf_get_lid_rejected_from_psd(d)),
+                     vcdinfo_ofs2str(obj, vcdinf_get_prev_from_psd(d), ext),
+                     vcdinfo_ofs2str(obj, vcdinf_get_next_from_psd(d), ext),
+                     vcdinfo_ofs2str(obj, vcdinf_get_return_from_psd(d),ext),
+                     vcdinfo_ofs2str(obj, vcdinf_get_default_from_psd(d), ext),
+                     vcdinfo_ofs2str(obj, vcdinfo_get_timeout_LID(d), ext),
                      vcdinfo_get_timeout_time(d),
-                     vcdinfo_get_loop_count(d), _vcd_bool_str (0x80 & d->loop),
-                     vcdinfo_pin2str (vcdinfo_get_itemid_from_psd(d)));
+                     vcdinf_get_loop_count(d), 
+                     _vcd_bool_str (vcdinf_has_jump_delay(d)),
+                     vcdinfo_pin2str (vcdinf_get_itemid_from_psd(d)));
 
-            for (i = 0; i < d->nos; i++)
+            for (i = 0; i < vcdinf_get_num_selections(d); i++)
               fprintf (stdout, "  ofs[%d]: %s\n", i,
                        vcdinfo_ofs2str (obj, 
-                                        vcdinfo_get_offset_from_psd(d, i), 
+                                        vcdinf_get_offset_from_psd(d, i), 
                                         ext));
 
             if (type == PSD_TYPE_EXT_SELECTION_LIST 
@@ -385,7 +392,7 @@ dump_psd (const vcdinfo_obj_t *obj, bool ext)
                          vcdinfo_area_str (&d2->return_area),
                          vcdinfo_area_str (&d2->default_area));
 
-                for (i = 0; i < d->nos; i++)
+                for (i = 0; i < vcdinf_get_num_selections(d); i++)
                   fprintf (stdout, "  area[%d]: %s\n", i,
                            vcdinfo_area_str (&d2->area[i]));
               }
@@ -743,8 +750,8 @@ dump_scandata_dat (const vcdinfo_obj_t *obj)
         (const void *) &_sd3->mpeg_track_offsets[track_count];
 
       const int scandata_ofs0 = 
-        offsetof (ScandataDat3, mpeg_track_offsets[track_count])
-        - offsetof (ScandataDat3, mpeg_track_offsets);
+        __vcd_offsetof (ScandataDat3, mpeg_track_offsets[track_count])
+        - __vcd_offsetof (ScandataDat3, mpeg_track_offsets);
 
       int n;
 
@@ -1092,9 +1099,11 @@ dump (char image_fname[])
       gl.source_type = VCDINFO_SOURCE_DEVICE;
     }
   
-  open_rc = vcdinfo_open(&obj, image_fname, gl.source_type);
-  if (open_rc==VCDINFO_OPEN_ERROR)
+  open_rc = vcdinfo_open(&obj, &image_fname, gl.source_type, gl.access_mode);
+  if (open_rc==VCDINFO_OPEN_ERROR) {
+    vcd_error ("Error determining place to read from");
     exit (EXIT_FAILURE);
+  }
 
   vcd_assert (obj.img != NULL);
 
@@ -1143,7 +1152,7 @@ dump (char image_fname[])
 
   psd_size = vcdinfo_get_psd_size (&obj);
 
-  if (vcdinfo_read_psd (&obj) && vcdinfo_read_psd (&obj))
+  if (vcdinfo_read_psd (&obj))
     {
       /* ISO9660 crosscheck */
       if (vcd_image_source_fs_stat (obj.img, 
@@ -1158,7 +1167,7 @@ dump (char image_fname[])
           if (psd_size != statbuf.size)
             vcd_warn ("ISO9660 psd size != INFO psd size");
           if (statbuf.lsn != PSD_VCD_SECTOR)
-            vcd_warn ("psd fileentry in ISO9660 not at fixed lsn");
+            vcd_warn ("psd file entry in ISO9660 not at fixed LSN");
         }
           
     }
@@ -1280,11 +1289,13 @@ enum {
   OP_SOURCE_AUTO        = VCDINFO_SOURCE_AUTO,
   OP_SOURCE_BIN         = VCDINFO_SOURCE_BIN,
   OP_SOURCE_CUE         = VCDINFO_SOURCE_CUE,
+  OP_SOURCE_NRG         = VCDINFO_SOURCE_NRG,
   OP_SOURCE_DEVICE      = VCDINFO_SOURCE_DEVICE,
   OP_SOURCE_SECTOR_2336 = VCDINFO_SOURCE_SECTOR_2336,
 
   /* These are the remaining configuration options */
   OP_VERSION,       OP_ENTRIES,   OP_INFO,      OP_PVD,       OP_SHOW, 
+  OP_ACCESS_MODE
 
 };
 
@@ -1295,6 +1306,7 @@ init()
   gl.verbose_flag     = false;
   gl.quiet_flag       = false;
   gl.source_type      = VCDINFO_SOURCE_UNDEF;
+  gl.access_mode      = NULL;
 
   /* Set all of show-flag entries false in one go. */
   memset(&gl.show, false, sizeof(gl.show));
@@ -1372,15 +1384,23 @@ main (int argc, const char *argv[])
   /* Command-line options */
   struct poptOption optionsTable[] = {
 
+    {"access-mode", 'a', 
+     POPT_ARG_STRING, &gl.access_mode, 
+     OP_ACCESS_MODE,
+     "set CD-ROM access mode (IOCTL, READ_10, READ_CD)", "ACCESS"},
+
     {"bin-file", 'b', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name, 
-     OP_SOURCE_BIN, "set \"bin\" image file as source", "FILE"},
+     OP_SOURCE_BIN, "set \"bin\" CD-ROM disk image file as source", "FILE"},
 
     {"cue-file", 'c', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name, 
-     OP_SOURCE_CUE, "set \"cue\" image file as source", "FILE"},
+     OP_SOURCE_CUE, "set \"cue\" CD-ROM disk image file as source", "FILE"},
 
     {"input", 'i', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name, 
      OP_SOURCE_AUTO,
-     "set source and determine if \"bin\" image or device", "INPUT"},
+     "set source and determine if \"bin\" image or device", "FILE"},
+
+    {"nrg-file", 'N', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &source_name, 
+     OP_SOURCE_NRG, "set Nero CD-ROM disk image file as source", "FILE"},
 
     {"sector-2336", '\0', 
      POPT_ARG_NONE, &sector_2336_flag, 
@@ -1543,9 +1563,14 @@ main (int argc, const char *argv[])
         exit (EXIT_SUCCESS);
         break;
 
+      case OP_ACCESS_MODE:
+        /* Make sure a we do only once? */
+        break;
+
       case OP_SOURCE_AUTO:
       case OP_SOURCE_BIN: 
       case OP_SOURCE_CUE: 
+      case OP_SOURCE_NRG: 
       case OP_SOURCE_DEVICE: 
       case OP_SOURCE_SECTOR_2336:
         {
@@ -1571,6 +1596,7 @@ main (int argc, const char *argv[])
             break;
           case OP_SOURCE_AUTO:
           case OP_SOURCE_CUE:
+          case OP_SOURCE_NRG: 
           case OP_SOURCE_DEVICE:
             /* This case is implied, but we'll make it explicit anyway. */
             okay = false;
