@@ -1,7 +1,7 @@
 /*
-    $Id: vcd_xml_rip.c,v 1.38 2003/01/06 17:19:59 hvr Exp $
+    $Id: vcd_xml_rip.c,v 1.52 2003/03/11 05:24:21 rocky Exp $
 
-    Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
+    Copyright (C) 2001,2003 Herbert Valerio Riedel <hvr@gnu.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,11 +22,21 @@
 # include "config.h"
 #endif
 
-#include <ctype.h>
-#include <stdlib.h>
-#include <string.h>
 #include <stdio.h>
+#include <errno.h>
+#include <ctype.h>
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
 #include <popt.h>
 
@@ -46,39 +56,19 @@
 #include <libvcd/vcd_types.h>
 #include <libvcd/vcd_util.h>
 #include <libvcd/vcd_xa.h>
+#include <libvcd/vcdinfo.h>
+#include <libvcd/vcdinf.h>
 
 #include "vcdxml.h"
 #include "vcd_xml_dtd.h"
 #include "vcd_xml_dump.h"
 #include "vcd_xml_common.h"
 
-static const char _rcsid[] = "$Id: vcd_xml_rip.c,v 1.38 2003/01/06 17:19:59 hvr Exp $";
+static const char _rcsid[] = "$Id: vcd_xml_rip.c,v 1.52 2003/03/11 05:24:21 rocky Exp $";
 
 static int _verbose_flag = 0;
 static int _quiet_flag = 0;
 
-static const char *
-_strip_trail (const char str[], size_t n)
-{
-  static char buf[1024];
-  int j;
-
-  vcd_assert (n < 1024);
-
-  strncpy (buf, str, n);
-  buf[n] = '\0';
-
-  for (j = strlen (buf) - 1; j >= 0; j--)
-    {
-      if (buf[j] != ' ')
-        break;
-
-      buf[j] = '\0';
-    }
-
-  return buf;
-}
-
 static void
 _register_file (struct vcdxml_t *obj, const char *pathname,
 		vcd_image_stat_t const *statbuf)
@@ -213,7 +203,12 @@ _parse_pvd (struct vcdxml_t *obj, VcdImageSource *img)
   memset (&pvd, 0, sizeof (struct iso_primary_descriptor));
   vcd_assert (sizeof (struct iso_primary_descriptor) == ISO_BLOCKSIZE);
 
-  vcd_image_source_read_mode2_sector (img, &pvd, ISO_PVD_SECTOR, false);
+  if (vcd_image_source_read_mode2_sector (img, &pvd, ISO_PVD_SECTOR, false) != 0) 
+    {
+      vcd_error ("error reading PVD sector (%d)", ISO_PVD_SECTOR);
+      return -1;
+    }
+  
 
   if (pvd.type != ISO_VD_PRIMARY)
     {
@@ -228,12 +223,11 @@ _parse_pvd (struct vcdxml_t *obj, VcdImageSource *img)
       return -1;
     }
  
-  obj->pvd.volume_id = strdup (_strip_trail (pvd.volume_id, 32));
-  obj->pvd.system_id = strdup (_strip_trail (pvd.system_id, 32));
-
-  obj->pvd.publisher_id = strdup (_strip_trail (pvd.publisher_id, 128));
-  obj->pvd.preparer_id = strdup (_strip_trail (pvd.preparer_id, 128));
-  obj->pvd.application_id = strdup (_strip_trail (pvd.application_id, 128));
+  obj->pvd.volume_id      = strdup (vcdinf_get_volume_id(&pvd));
+  obj->pvd.system_id      = strdup (vcdinf_get_system_id(&pvd));
+  obj->pvd.publisher_id   = strdup (vcdinf_get_publisher_id(&pvd));
+  obj->pvd.preparer_id    = strdup (vcdinf_get_preparer_id(&pvd));
+  obj->pvd.application_id = strdup (vcdinf_get_application_id(&pvd));
 
   return 0;
 }
@@ -256,19 +250,11 @@ _parse_info (struct vcdxml_t *obj, VcdImageSource *img)
   switch (_vcd_type)
     {
     case VCD_TYPE_VCD:
-      vcd_debug ("VCD 1.0 detected\n");
-      break;
     case VCD_TYPE_VCD11:
-      vcd_debug ("VCD 1.1 detected");
-      break;
     case VCD_TYPE_VCD2:
-      vcd_debug ("VCD 2.0 detected");
-      break;
     case VCD_TYPE_SVCD:
-      vcd_debug ("SVCD detected");
-      break;
     case VCD_TYPE_HQVCD:
-      vcd_debug ("HQ-VCD detected\n");
+      vcd_debug ("%s detected", vcdinf_get_format_version_str(_vcd_type));
       break;
     case VCD_TYPE_INVALID:
       vcd_error ("unknown ID encountered -- maybe not a proper (S)VCD?");
@@ -279,33 +265,33 @@ _parse_info (struct vcdxml_t *obj, VcdImageSource *img)
       break;
     }
 
-  obj->info.album_id = strdup (_strip_trail (info.album_desc, 16));
-  obj->info.volume_count = uint16_from_be (info.vol_count);
-  obj->info.volume_number = uint16_from_be (info.vol_id);
+  obj->info.album_id = strdup (vcdinf_get_album_id(&info));
+  obj->info.volume_count = vcdinf_get_volume_count(&info);
+  obj->info.volume_number = vcdinf_get_volume_num(&info);
 
   obj->info.restriction = info.flags.restriction;
   obj->info.use_lid2 = info.flags.use_lid2;
   obj->info.use_sequence2 = info.flags.use_track3;
 
-  obj->info.psd_size = uint32_from_be (info.psd_size);
-  obj->info.max_lid = uint16_from_be (info.lot_entries);
+  obj->info.psd_size = vcdinf_get_psd_size(&info);
+  obj->info.max_lid = vcdinf_get_num_LIDs(&info);
 
   {
-    unsigned segment_start;
+    lba_t segment_start;
     unsigned max_seg_num;
     int idx, n;
     struct segment_t *_segment = NULL;
 
     segment_start = msf_to_lba (&info.first_seg_addr);
 
-    max_seg_num = uint16_from_be (info.item_count);
+    max_seg_num = vcdinf_get_num_segments(&info);
 
-    if (segment_start < 150)
+    if (segment_start < PREGAP_SECTORS)
       return 0;
 
-    segment_start -= 150;
+    segment_start = vcdinfo_lba2lsn(segment_start);
 
-    vcd_assert (segment_start % 75 == 0);
+    vcd_assert (segment_start % CD_FRAMES_PER_SECOND == 0);
 
     obj->info.segments_start = segment_start;
 
@@ -366,15 +352,15 @@ _parse_entries (struct vcdxml_t *obj, VcdImageSource *img)
     }
 
   ltrack = 0;
-  for (idx = 0; idx < uint16_from_be (entries.entry_count); idx++)
+  for (idx = 0; idx < vcdinf_get_num_entries(&entries); idx++)
     {
-      uint32_t extent = msf_to_lba(&(entries.entry[idx].msf));
-      uint8_t track = from_bcd8 (entries.entry[idx].n);
+      lba_t extent = vcdinf_get_entry_lba(&entries, idx);
+      track_t track = vcdinf_get_track(&entries, idx);
       bool newtrack = (track != ltrack);
       ltrack = track;
       
-      vcd_assert (extent >= 150);
-      extent -= 150;
+      vcd_assert (extent >= PREGAP_SECTORS);
+      extent = vcdinfo_lba2lsn(extent);
 
       vcd_assert (track >= 2);
       track -= 2;
@@ -432,13 +418,6 @@ _xstrdup(const char *s)
   return NULL;
 }
 
-typedef struct {
-  uint8_t type;
-  unsigned lid;
-  unsigned offset;
-  bool in_lot;
-} psd_offset_t;
-
 struct _pbc_ctx {
   unsigned psd_size;
   unsigned maximum_lid;
@@ -454,21 +433,26 @@ static const char *
 _pin2id (unsigned pin, const struct _pbc_ctx *_ctx)
 {
   static char buf[80];
+  vcdinfo_itemid_t itemid;
+  
+  vcdinfo_classify_itemid (pin, &itemid);
 
-  if (pin < 2)
+  switch(itemid.type) {
+  case VCDINFO_ITEM_TYPE_NOTFOUND:
     return NULL;
-
-  if (pin < 100)
-    snprintf (buf, sizeof (buf), "sequence-%2.2d", pin - 2);
-  else if (pin < 600)
-    snprintf (buf, sizeof (buf), "entry-%3.3d", pin - 100);
-  else if (pin < 1000)
+  case VCDINFO_ITEM_TYPE_TRACK:
+    snprintf (buf, sizeof (buf), "sequence-%2.2d", itemid.num-1);
+    break;
+  case VCDINFO_ITEM_TYPE_ENTRY:
+    snprintf (buf, sizeof (buf), "entry-%3.3d", itemid.num);
+    break;
+  case VCDINFO_ITEM_TYPE_SEGMENT:
+    snprintf (buf, sizeof (buf), "segment-%4.4d", itemid.num+1);
+    break;
+  case VCDINFO_ITEM_TYPE_SPAREID2:
+  default:
     return NULL;
-  else if (pin < 2980)
-    snprintf (buf, sizeof (buf), "segment-%4.4d", pin - 1000);
-  else 
-    return NULL;
-
+  }
   return buf;
 }
 
@@ -478,7 +462,7 @@ _ofs2id (unsigned offset, const struct _pbc_ctx *_ctx)
   VcdListNode *node;
   static char buf[80];
   unsigned sl_num = 0, el_num = 0, pl_num = 0;
-  psd_offset_t *ofs = NULL;
+  vcdinfo_offset_t *ofs = NULL;
   
   if (offset == PSD_OFS_DISABLED)
     return NULL;
@@ -539,25 +523,12 @@ _ofs2id (unsigned offset, const struct _pbc_ctx *_ctx)
   return buf;
 }
 
-static int
-_calc_time (uint16_t wtime)
-{
-  if (wtime < 61)
-    return wtime;
-  else if (wtime < 255)
-    return (wtime - 60) * 10 + 60;
-  else
-    return -1;
-}
-
-
-
 static pbc_t *
 _pbc_node_read (const struct _pbc_ctx *_ctx, unsigned offset)
 {
   pbc_t *_pbc = NULL;
   const uint8_t *_buf = &_ctx->psd[offset * _ctx->offset_mult];
-  psd_offset_t *ofs = NULL;
+  vcdinfo_offset_t *ofs = NULL;
 
   {
     VcdListNode *node;
@@ -579,18 +550,21 @@ _pbc_node_read (const struct _pbc_ctx *_ctx, unsigned offset)
       _pbc = vcd_pbc_new (PBC_PLAYLIST);
       {
 	const PsdPlayListDescriptor *d = (const void *) _buf;
-	_pbc->prev_id = _xstrdup (_ofs2id (uint16_from_be (d->prev_ofs), _ctx));
-	_pbc->next_id = _xstrdup (_ofs2id (uint16_from_be (d->next_ofs), _ctx));
-	_pbc->retn_id = _xstrdup (_ofs2id (uint16_from_be (d->return_ofs), _ctx));
+	_pbc->prev_id = _xstrdup (_ofs2id (vcdinf_get_prev_from_pld(d), 
+					   _ctx));
+	_pbc->next_id = _xstrdup (_ofs2id (vcdinf_get_next_from_pld(d),
+					   _ctx));
+	_pbc->retn_id = _xstrdup (_ofs2id (vcdinf_get_return_from_pld(d), 
+					   _ctx));
 	
-	_pbc->playing_time = (double) (uint16_from_be (d->ptime)) / 15.0;
-	_pbc->wait_time = _calc_time (d->wtime);
-	_pbc->auto_pause_time =  _calc_time(d->atime);
+	_pbc->playing_time = (double) (vcdinf_get_play_time(d)) / 15.0;
+	_pbc->wait_time       = vcdinfo_get_wait_time(d);
+	_pbc->auto_pause_time = vcdinfo_get_autowait_time(d);
 
-	for (n = 0; n < d->noi; n++)
-	  {
-	    _vcd_list_append (_pbc->item_id_list, 
-			      _xstrdup (_pin2id (uint16_from_be (d->itemid[n]), _ctx)));
+	for (n = 0; n < d->noi; n++) {
+	  _vcd_list_append (_pbc->item_id_list, 
+			    _xstrdup(_pin2id(vcdinf_get_play_item_from_pld(d, n),
+					     _ctx)));
 	  }
       }
       break;
@@ -600,12 +574,15 @@ _pbc_node_read (const struct _pbc_ctx *_ctx, unsigned offset)
       _pbc = vcd_pbc_new (PBC_SELECTION);
       {
 	const PsdSelectionListDescriptor *d = (const void *) _buf;
-	_pbc->bsn = d->bsn;
-	_pbc->prev_id = _xstrdup (_ofs2id (uint16_from_be (d->prev_ofs), _ctx));
-	_pbc->next_id = _xstrdup (_ofs2id (uint16_from_be (d->next_ofs), _ctx));
-	_pbc->retn_id = _xstrdup (_ofs2id (uint16_from_be (d->return_ofs), _ctx));
+	_pbc->bsn = vcdinf_get_bsn(d);
+	_pbc->prev_id = _xstrdup (_ofs2id (vcdinf_get_prev_from_psd(d), 
+					   _ctx));
+	_pbc->next_id = _xstrdup (_ofs2id (vcdinf_get_next_from_psd(d), 
+					   _ctx));
+	_pbc->retn_id = _xstrdup (_ofs2id (vcdinf_get_return_from_psd(d), 
+					   _ctx));
 
-	switch (uint16_from_be (d->default_ofs))
+	switch (vcdinf_get_default_from_psd(d))
 	  {
 	  case PSD_OFS_DISABLED:
 	    _pbc->default_id = NULL;
@@ -630,12 +607,12 @@ _pbc_node_read (const struct _pbc_ctx *_ctx, unsigned offset)
 
 	_pbc->timeout_id = 
 	  _xstrdup (_ofs2id (uint16_from_be (d->timeout_ofs), _ctx));
-	_pbc->timeout_time = _calc_time (d->totime);
-	_pbc->jump_delayed = (0x80 & d->loop) != 0;
-	_pbc->loop_count = (0x7f & d->loop);
+	_pbc->timeout_time = vcdinfo_get_timeout_time (d);
+	_pbc->jump_delayed = vcdinf_has_jump_delay(d);
+	_pbc->loop_count = vcdinf_get_loop_count(d);
 	_pbc->item_id = _xstrdup (_pin2id (uint16_from_be (d->itemid), _ctx));
 
-	for (n = 0; n < d->nos; n++)
+	for (n = 0; n < vcdinf_get_num_selections(d); n++)
 	  {
 	    _vcd_list_append (_pbc->select_id_list, 
 			      _xstrdup (_ofs2id (uint16_from_be (d->ofs[n]), _ctx)));
@@ -656,7 +633,7 @@ _pbc_node_read (const struct _pbc_ctx *_ctx, unsigned offset)
 	    *_pbc->return_area = d2->return_area;
 	    *_pbc->default_area = d2->default_area;
 
-	    for (n = 0; n < d->nos; n++)
+	    for (n = 0; n < vcdinf_get_num_selections(d); n++)
 	      {
 		pbc_area_t *_area = _vcd_malloc (sizeof (pbc_area_t));
 
@@ -682,7 +659,7 @@ _pbc_node_read (const struct _pbc_ctx *_ctx, unsigned offset)
       break;
 
     default:
-      vcd_warn ("unknown psd type %d", *_buf);
+      vcd_warn ("Unknown PSD type %d", *_buf);
       break;
     }
 
@@ -699,7 +676,7 @@ static void
 _visit_pbc (struct _pbc_ctx *obj, unsigned lid, unsigned offset, bool in_lot)
 {
   VcdListNode *node;
-  psd_offset_t *ofs;
+  vcdinfo_offset_t *ofs;
   unsigned _rofs = offset * obj->offset_mult;
 
   vcd_assert (obj->psd_size % 8 == 0);
@@ -743,7 +720,7 @@ _visit_pbc (struct _pbc_ctx *obj, unsigned lid, unsigned offset, bool in_lot)
         }
     }
 
-  ofs = _vcd_malloc (sizeof (psd_offset_t));
+  ofs = _vcd_malloc (sizeof (vcdinfo_offset_t));
 
   ofs->offset = offset;
   ofs->lid = lid;
@@ -765,9 +742,9 @@ _visit_pbc (struct _pbc_ctx *obj, unsigned lid, unsigned offset, bool in_lot)
             vcd_warn ("LOT entry assigned LID %d, but descriptor has LID %d",
                       ofs->lid, uint16_from_be (d->lid) & 0x7fff);
 
-        _visit_pbc (obj, 0, uint16_from_be (d->prev_ofs), false);
-        _visit_pbc (obj, 0, uint16_from_be (d->next_ofs), false);
-        _visit_pbc (obj, 0, uint16_from_be (d->return_ofs), false);
+        _visit_pbc (obj, 0, vcdinf_get_prev_from_pld(d), false);
+        _visit_pbc (obj, 0, vcdinf_get_next_from_pld(d), false);
+        _visit_pbc (obj, 0, vcdinf_get_return_from_pld(d), false);
       }
       break;
 
@@ -788,13 +765,13 @@ _visit_pbc (struct _pbc_ctx *obj, unsigned lid, unsigned offset, bool in_lot)
             vcd_warn ("LOT entry assigned LID %d, but descriptor has LID %d",
                       ofs->lid, uint16_from_be (d->lid) & 0x7fff);
 
-        _visit_pbc (obj, 0, uint16_from_be (d->prev_ofs), false);
-        _visit_pbc (obj, 0, uint16_from_be (d->next_ofs), false);
-        _visit_pbc (obj, 0, uint16_from_be (d->return_ofs), false);
-        _visit_pbc (obj, 0, uint16_from_be (d->default_ofs), false);
+        _visit_pbc (obj, 0, vcdinf_get_prev_from_psd(d), false);
+        _visit_pbc (obj, 0, vcdinf_get_next_from_psd(d), false);
+        _visit_pbc (obj, 0, vcdinf_get_return_from_psd(d), false);
+        _visit_pbc (obj, 0, vcdinf_get_default_from_psd(d), false);
         _visit_pbc (obj, 0, uint16_from_be (d->timeout_ofs), false);
 
-        for (idx = 0; idx < d->nos; idx++)
+        for (idx = 0; idx < vcdinf_get_num_selections(d); idx++)
           _visit_pbc (obj, 0, uint16_from_be (d->ofs[idx]), false);
         
       }
@@ -812,25 +789,6 @@ _visit_pbc (struct _pbc_ctx *obj, unsigned lid, unsigned offset, bool in_lot)
     }
 }
 
-static int
-_offset_t_cmp (psd_offset_t *a, psd_offset_t *b)
-{
-  if (a->lid && b->lid)
-    {
-      if (a->lid > b->lid)
-	return 1;
-
-      if (a->lid < b->lid)
-	return -1;
-    } 
-  else if (a->lid)
-    return -1;
-  else if (b->lid)
-    return 1;
-
-  return 0;
-}
-
 static void
 _visit_lot (struct _pbc_ctx *obj)
 {
@@ -841,7 +799,7 @@ _visit_lot (struct _pbc_ctx *obj)
     if ((tmp = uint16_from_be (lot->offset[n])) != PSD_OFS_DISABLED)
       _visit_pbc (obj, n + 1, tmp, true);
 
-  _vcd_list_sort (obj->offset_list, (_vcd_list_cmp_func) _offset_t_cmp);
+  _vcd_list_sort (obj->offset_list, (_vcd_list_cmp_func) vcdinf_lid_t_cmp);
 }
 
 static int
@@ -858,7 +816,7 @@ _parse_pbc (struct vcdxml_t *obj, VcdImageSource *img, bool no_ext_psd)
 
   if (!obj->info.psd_size)
     {
-      vcd_debug ("no pbc info");
+      vcd_debug ("No PBC info");
       return 0;
     }
 
@@ -926,7 +884,7 @@ _parse_pbc (struct vcdxml_t *obj, VcdImageSource *img, bool no_ext_psd)
 
   _VCD_LIST_FOREACH (node, _pctx.offset_list)
     {
-      psd_offset_t *ofs = _vcd_list_node_data (node);
+      vcdinfo_offset_t *ofs = _vcd_list_node_data (node);
       pbc_t *_pbc;
 
       vcd_assert (ofs->offset != PSD_OFS_DISABLED);
@@ -966,9 +924,11 @@ _rip_isofs (struct vcdxml_t *obj, VcdImageSource *img)
       for (idx = 0; idx < _fs->size; idx += blocksize)
 	{
 	  char buf[blocksize];
-	  memset (&buf, 0, sizeof (buf));
+	  /* some EDG based compilers segfault on the commented out code */
+	  /* memset (buf, 0, sizeof (buf)); */
+	  memset (buf, 0, blocksize);
 
-	  vcd_image_source_read_mode2_sector (img, &buf, 
+	  vcd_image_source_read_mode2_sector (img, buf, 
 					      _fs->lsn + (idx / blocksize),
 					      _fs->file_raw);
 
@@ -1000,7 +960,7 @@ _rip_segments (struct vcdxml_t *obj, VcdImageSource *img)
 
   start_extent = obj->info.segments_start;
 
-  vcd_assert (start_extent % 75 == 0);
+  vcd_assert (start_extent % CD_FRAMES_PER_SECOND == 0);
 
   _VCD_LIST_FOREACH (node, obj->segment_list)
     {
@@ -1028,7 +988,7 @@ _rip_segments (struct vcdxml_t *obj, VcdImageSource *img)
 	  struct m2f2sector
           {
             uint8_t subheader[8];
-            uint8_t data[2324];
+            uint8_t data[M2F2_SECTOR_SIZE];
             uint8_t spare[4];
           }
           buf;
@@ -1046,7 +1006,7 @@ _rip_segments (struct vcdxml_t *obj, VcdImageSource *img)
               break;
             }
 
-	  vcd_mpeg_parse_packet (buf.data, 2324, false, &mpeg_ctx);
+	  vcd_mpeg_parse_packet (buf.data, M2F2_SECTOR_SIZE, false, &mpeg_ctx);
 
 	  if (mpeg_ctx.packet.has_pts)
 	    {
@@ -1068,7 +1028,7 @@ _rip_segments (struct vcdxml_t *obj, VcdImageSource *img)
 	      _vcd_list_append (_seg->autopause_list, _ap_ts);
 	    }
 
-	  fwrite (buf.data, 2324, 1, outfd);
+	  fwrite (buf.data, M2F2_SECTOR_SIZE, 1, outfd);
 
 	  if (ferror (outfd))
             {
@@ -1089,9 +1049,10 @@ _rip_segments (struct vcdxml_t *obj, VcdImageSource *img)
 }
 
 static int
-_rip_sequences (struct vcdxml_t *obj, VcdImageSource *img)
+_rip_sequences (struct vcdxml_t *obj, VcdImageSource *img, int tracknum)
 {
   VcdListNode *node;
+  int counter=1;
 
   _VCD_LIST_FOREACH (node, obj->sequence_list)
     {
@@ -1109,11 +1070,16 @@ _rip_sequences (struct vcdxml_t *obj, VcdImageSource *img)
       struct m2f2sector
       {
 	uint8_t subheader[8];
-	uint8_t data[2324];
+	uint8_t data[M2F2_SECTOR_SIZE];
 	uint8_t spare[4];
       }
       buf[15];
 
+      if (tracknum && tracknum!=counter++) {
+	vcd_info("Track %d selected, skipping track %d", tracknum,counter-1);
+	continue;
+      }
+	  
       memset (&mpeg_ctx, 0, sizeof (VcdMpegStreamCtx));
 
       start_lsn = _seq->start_extent;
@@ -1197,7 +1163,8 @@ _rip_sequences (struct vcdxml_t *obj, VcdImageSource *img)
 	    {
 	      VcdListNode *_node;
 
-	      vcd_mpeg_parse_packet (buf[buf_idx].data, 2324, false, &mpeg_ctx);
+	      vcd_mpeg_parse_packet (buf[buf_idx].data, M2F2_SECTOR_SIZE, 
+				     false, &mpeg_ctx);
 
 	      if (!mpeg_ctx.packet.zero)
 		last_nonzero = n;
@@ -1238,7 +1205,7 @@ _rip_sequences (struct vcdxml_t *obj, VcdImageSource *img)
 	      
 	      if (first_data)
 		{
-		  fwrite (buf[buf_idx].data, 2324, 1, outfd);
+		  fwrite (buf[buf_idx].data, M2F2_SECTOR_SIZE, 1, outfd);
 
 		  if (ferror (outfd))
 		    {
@@ -1271,7 +1238,7 @@ _rip_sequences (struct vcdxml_t *obj, VcdImageSource *img)
 	  vcd_debug ("truncating file to %d packets", length);
 
 	  fflush (outfd);
-	  if (ftruncate (fileno (outfd), length * 2324))
+	  if (ftruncate (fileno (outfd), length * M2F2_SECTOR_SIZE))
 	    perror ("ftruncate()");
 	}
 
@@ -1301,13 +1268,16 @@ main (int argc, const char *argv[])
   int sector_2336_flag = 0;
   int _progress_flag = 0;
   int _gui_flag = 0;
+  int _track_flag=0;
 
   enum { 
-    CL_VERSION = 1,
-    CL_BINCUE,
-    CL_CDROM,
-    CL_NRG
-  } _img_type = 0;
+    CL_SOURCE_UNDEF = VCDINFO_SOURCE_UNDEF,
+    CL_SOURCE_AUTO  = VCDINFO_SOURCE_AUTO,
+    CL_SOURCE_BIN   = VCDINFO_SOURCE_BIN,
+    CL_SOURCE_CDROM = VCDINFO_SOURCE_DEVICE,
+    CL_SOURCE_NRG   = VCDINFO_SOURCE_NRG,
+    CL_VERSION      = 20
+  } _img_type = CL_SOURCE_UNDEF;
 
   vcd_xml_progname = "vcdxrip";
 
@@ -1326,18 +1296,23 @@ main (int argc, const char *argv[])
        "specify xml file for output (default: '" DEFAULT_XML_FNAME "')",
        "FILE"},
 
-      {"bin-file", 'b', POPT_ARG_STRING, &img_fname, CL_BINCUE,
+      {"bin-file", 'b', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &img_fname, 
+       CL_SOURCE_BIN,
        "set image file as source (default: '" DEFAULT_IMG_FNAME "')", 
        "FILE"},
 
       {"sector-2336", '\0', POPT_ARG_NONE, &sector_2336_flag, 0,
        "use 2336 byte sector mode for image file"},
 
-      {"cdrom-device", '\0', POPT_ARG_STRING, &img_fname, CL_CDROM,
-       "set CDROM device as source", "DEVICE"},
+      {"cdrom-device", 'C', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &img_fname,
+       CL_SOURCE_CDROM,"set CDROM device as source", "DEVICE"},
 
-      {"nrg-file", '\0', POPT_ARG_STRING, &img_fname, CL_NRG,
-       "set NRG image file as source",
+      {"input", 'i', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &img_fname, 
+       CL_SOURCE_AUTO, 
+       "set source and determine if \"bin\" image or device", "FILE"},
+
+      {"nrg-file", 'N', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, &img_fname, 
+       CL_SOURCE_NRG, "set Nero CD-ROM disk image image file as source",
        "FILE"},
 
       {"no-ext-psd", '\0', POPT_ARG_NONE, &no_ext_psd_flag, 0,
@@ -1368,6 +1343,9 @@ main (int argc, const char *argv[])
 
       {"version", 'V', POPT_ARG_NONE, NULL, CL_VERSION,
        "display version and copyright information and exit"},
+	   
+      { "track", 't', POPT_ARG_INT, &_track_flag, 0,
+	"rip only this track"},
       
       POPT_AUTOHELP {NULL, 0, 0, NULL, 0}
     };
@@ -1386,19 +1364,23 @@ main (int argc, const char *argv[])
 	  exit (EXIT_SUCCESS);
 	  break;
 
-	case CL_NRG:
-	case CL_BINCUE:
-	case CL_CDROM:
+	case CL_SOURCE_AUTO:
+	case CL_SOURCE_NRG:
+	case CL_SOURCE_BIN:
+	case CL_SOURCE_CDROM:
 	  if (_img_type)
 	    {
-	      vcd_error ("only one image (type) supported at once - try --help");
+	      vcd_error ("Only one image (type) supported at once - try --help");
 	      exit (EXIT_FAILURE);
 	    }
 	  _img_type = opt;
 	  break;
 
 	default:
-	  vcd_error ("error while parsing command line - try --help");
+	  fprintf (stderr, "%s: %s\n", 
+		   poptBadOption(optCon, POPT_BADOPTION_NOALIAS),
+		   poptStrerror(opt));
+	  fprintf (stderr, "error while parsing command line - try --help\n");
 	  exit (EXIT_FAILURE);
 	  break;
       }
@@ -1407,7 +1389,7 @@ main (int argc, const char *argv[])
       vcd_error ("I can't be both, quiet and verbose... either one or another ;-)");
     
     if (poptGetArgs (optCon) != NULL)
-      vcd_error ("why are you giving me non-option arguments? -- try --help");
+      vcd_error ("Why are you giving me non-option arguments? -- try --help");
 
     poptFreeContext (optCon);
   }
@@ -1428,40 +1410,14 @@ main (int argc, const char *argv[])
   if (!xml_fname)
     xml_fname = strdup (DEFAULT_XML_FNAME);
 
-  switch (_img_type) 
-    {
-    case CL_BINCUE:
-      img_src = vcd_image_source_new_bincue ();
-
-      vcd_image_source_set_arg (img_src, "bin", img_fname);
-
-      vcd_image_source_set_arg (img_src, "sector", 
-				sector_2336_flag ? "2336" : "2352");
-      break;
-
-    case CL_NRG:
-      {
-	VcdDataSource *bin_source;
-	
-	bin_source = vcd_data_source_new_stdio (img_fname);
-	vcd_assert (bin_source != NULL);
-
-	img_src = vcd_image_source_new_nrg (bin_source);
-      }
-      break;
-
-    case CL_CDROM:
-      img_src = vcd_image_source_new_cd ();
-
-      vcd_image_source_set_arg (img_src, "device", img_fname);
-      break;
-
-    default:
-      vcd_error ("no image given - try --help");
-      exit (EXIT_FAILURE);
-      break;
-    }
+  if (CL_SOURCE_UNDEF == _img_type) 
+    _img_type = CL_SOURCE_AUTO;
 
+  if (!vcdinf_open(&img_src, &img_fname, _img_type, NULL)) {
+    vcd_error ("Error determining place to read from.");
+    exit (EXIT_FAILURE);
+  }
+  
   vcd_assert (img_src != NULL);
 
   /* start with ISO9660 PVD */
@@ -1476,10 +1432,12 @@ main (int argc, const char *argv[])
   /* needs to be parsed last! */
   _parse_pbc (&obj, img_src, no_ext_psd_flag);
 
-  if (norip_flag)
-    vcd_warn ("fyi, entry point and auto pause locations cannot be "
-	      "determined without mpeg extraction -- hope that's ok");
-  else
+  if (norip_flag || noseq_flag || noseg_flag || _track_flag) {
+    vcd_warn ("Entry offsets inside sequence-items may incorrect...");
+    vcd_warn ("and auto-pause locations might not be checked.");
+  }
+
+  if (!norip_flag)
     {
       if (!nofile_flag)
 	_rip_isofs (&obj, img_src);
@@ -1488,10 +1446,10 @@ main (int argc, const char *argv[])
 	_rip_segments (&obj, img_src);
 
       if (!noseq_flag)
-	_rip_sequences (&obj, img_src);
+	_rip_sequences (&obj, img_src, _track_flag);
     }
 
-  vcd_info ("writing xml description to `%s'...", xml_fname);
+  vcd_info ("Writing XML description to `%s'...", xml_fname);
   vcd_xml_dump (&obj, xml_fname);
 
   vcd_image_source_destroy (img_src);
