@@ -1,5 +1,5 @@
 /*
-    $Id: vcd.c,v 1.95 2003/02/09 14:19:32 rocky Exp $
+    $Id: vcd.c,v 1.96 2003/03/04 14:35:20 hvr Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -46,7 +46,7 @@
 #include <libvcd/vcd_dict.h>
 #include <libvcd/vcd_image.h>
 
-static const char _rcsid[] = "$Id: vcd.c,v 1.95 2003/02/09 14:19:32 rocky Exp $";
+static const char _rcsid[] = "$Id: vcd.c,v 1.96 2003/03/04 14:35:20 hvr Exp $";
 
 static const char zero[CDDA_SECTOR_SIZE] = { 0, };
 
@@ -2369,11 +2369,12 @@ vcd_version_string (bool full_text)
     return ("GNU VCDImager " VERSION " [" HOST_ARCH "]");
 
   return ("%s (GNU VCDImager) " VERSION "\n"
-          "Written by Herbert Valerio Riedel\n"
+          "Written by Herbert Valerio Riedel and Rocky Bernstein.\n"
           "\n"
           "http://www.gnu.org/software/vcdimager/\n"
           "\n"
-          "Copyright (C) 2000, 2001 Herbert Valerio Riedel <hvr@gnu.org>\n"
+          "Copyright (C) 2000-2003 Herbert Valerio Riedel <hvr@gnu.org>\n"
+	  "                   2003 Rocky Bernstein <rocky@panix.com>\n"
           "\n"
           "This is free software; see the source for copying conditions.  There is NO\n"
           "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n");
