@@ -1,5 +1,5 @@
 /*
-    $Id: vcd_files.c,v 1.51 2001/11/27 17:33:51 hvr Exp $
+    $Id: vcd_files.c,v 1.53 2003/02/19 22:19:48 rocky Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -38,7 +38,7 @@
 
 #include <libvcd/vcd_pbc.h>
 
-static const char _rcsid[] = "$Id: vcd_files.c,v 1.51 2001/11/27 17:33:51 hvr Exp $";
+static const char _rcsid[] = "$Id: vcd_files.c,v 1.53 2003/02/19 22:19:48 rocky Exp $";
 
 inline static bool
 _pal_p (const struct vcd_mpeg_stream_vid_info *_info)
@@ -194,7 +194,7 @@ set_entries_vcd (VcdObj *obj, void *buf)
       lsect += obj->iso_size;
 
       entries_vcd.entry[idx].n = to_bcd8(track_idx);
-      lba_to_msf(lsect + 150, &(entries_vcd.entry[idx].msf));
+      lba_to_msf(_vcd_lsn_to_lba(lsect), &(entries_vcd.entry[idx].msf));
 
       idx++;
       lsect += obj->track_front_margin;
@@ -793,8 +793,8 @@ get_scandata_dat_size (const VcdObj *obj)
   retval += sizeof (msf_t) * _vcd_list_length (obj->mpeg_track_list);
 
   /* struct 2 */
-  vcd_assert (sizeof (ScandataDat2) == 0);
-  retval += sizeof (ScandataDat2);
+  /* vcd_assert (sizeof (ScandataDat2) == 0);
+     retval += sizeof (ScandataDat2); */
   retval += sizeof (uint16_t) * 0;
 
   /* struct 3 */
@@ -802,8 +802,8 @@ get_scandata_dat_size (const VcdObj *obj)
   retval += (sizeof (uint8_t) + sizeof (uint16_t)) * _vcd_list_length (obj->mpeg_track_list);
 
   /* struct 4 */
-  vcd_assert (sizeof (ScandataDat4) == 0);
-  retval += sizeof (ScandataDat4);
+  /* vcd_assert (sizeof (ScandataDat4) == 0);
+     retval += sizeof (ScandataDat4); */
   {
     VcdListNode *node;
     _VCD_LIST_FOREACH (node, obj->mpeg_track_list)
@@ -812,7 +812,6 @@ get_scandata_dat_size (const VcdObj *obj)
         
         retval += sizeof (msf_t) * _get_scandata_count (track->info);
       }
-    
   }
 
   return retval;
@@ -832,8 +831,8 @@ set_scandata_dat (VcdObj *obj, void *buf)
     (ScandataDat4 *) &(scandata_dat3->mpeg_track_offsets[tracks]);
 
   const uint16_t _begin_offset =
-    offsetof (ScandataDat3, mpeg_track_offsets[tracks])
-    - offsetof (ScandataDat3, mpeg_track_offsets);
+    __vcd_offsetof (ScandataDat3, mpeg_track_offsets[tracks])
+    - __vcd_offsetof (ScandataDat3, mpeg_track_offsets);
 
   VcdListNode *node;
   unsigned n;
@@ -933,7 +932,8 @@ vcd_files_info_detect_type (const void *info_buf)
       {
       case INFO_VERSION_VCD2:
         if (_info->sys_prof_tag != INFO_SPTAG_VCD2)
-          vcd_warn ("INFO.VCD: unexpected system profile tag encountered");
+          vcd_warn ("INFO.VCD: unexpected system profile tag %d encountered",
+                    _info->version);
         _type = VCD_TYPE_VCD2;
         break;
 
@@ -948,13 +948,15 @@ vcd_files_info_detect_type (const void *info_buf)
             _type = VCD_TYPE_VCD11;
             break;
           default:
-            vcd_warn ("INFO.VCD: unexpected system profile tag encountered, assuming VCD 1.1");
+            vcd_warn ("INFO.VCD: unexpected system profile tag %d "
+                      "encountered, assuming VCD 1.1", _info->sys_prof_tag);
             break;
           }
         break;
 
       default:
-        vcd_warn ("unexpected vcd version encountered -- assuming vcd 2.0");
+        vcd_warn ("unexpected VCD version %d encountered -- assuming VCD 2.0",
+                  _info->version);
         break;
       }
   else if (!strncmp (_info->ID, INFO_ID_SVCD, sizeof (_info->ID)))
@@ -962,12 +964,14 @@ vcd_files_info_detect_type (const void *info_buf)
       {
       case INFO_VERSION_SVCD:
         if (_info->sys_prof_tag != INFO_SPTAG_SVCD)
-          vcd_warn ("INFO.SVD: unexpected system profile tag value -- assuming svcd");
+          vcd_warn ("INFO.SVD: unexpected system profile tag value %d "
+                    "-- assuming SVCD", _info->sys_prof_tag);
         _type = VCD_TYPE_SVCD;
         break;
         
       default:
-        vcd_warn ("INFO.SVD: unexpected version value seen -- still assuming svcd");
+        vcd_warn ("INFO.SVD: unexpected version value %d seen "
+                  " -- still assuming SVCD", _info->version);
         _type = VCD_TYPE_SVCD;
         break;
       }
@@ -981,7 +985,8 @@ vcd_files_info_detect_type (const void *info_buf)
         break;
         
       default:
-        vcd_warn ("INFO.SVD: unexpected version value seen -- still assuming hqvcd");
+        vcd_warn ("INFO.SVD: unexpected version value %d seen "
+                  "-- still assuming HQVCD", _info->version);
         _type = VCD_TYPE_HQVCD;
         break;
       }
