@@ -1,5 +1,5 @@
 /*
-    $Id: vcd_files_private.h,v 1.30 2002/12/25 03:06:00 hvr Exp $
+    $Id: vcd_files_private.h,v 1.34 2003/02/23 19:59:46 hvr Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
               (C) 2000 Jens B. Jorgensen <jbj1@ultraemail.net>
@@ -47,6 +47,8 @@
 #define ENTRIES_VERSION_HQVCD 0x01
 #define ENTRIES_SPTAG_HQVCD   0x00
 
+PRAGMA_BEGIN_PACKED
+
 typedef struct {
   char ID[8];                             /* "ENTRYVCD" */
   uint8_t version;                        /* 0x02 --- VCD2.0
@@ -90,38 +92,38 @@ typedef struct {
 /* This one-byte field describes certain characteristics of the disc */
 typedef struct {
 #if defined(BITFIELD_LSBF)
-  uint8_t reserved1 : 1;                   /* Reserved, must be zero */
-  uint8_t restriction : 2;                 /* restriction, eg. "unsuitable
+  bool       reserved1 : 1;                   /* Reserved, must be zero */
+  bitfield_t restriction : 2;                 /* restriction, eg. "unsuitable
                                               for kids":
                                               0x0 ==> unrestricted,
                                               0x1 ==> restricted category 1,
                                               0x2 ==> restricted category 2,
                                               0x3 ==> restricted category 3 */
-  uint8_t special_info : 1;                /* Special Information is encoded 
+  bool       special_info : 1;                /* Special Information is encoded 
                                               in the pictures */
-  uint8_t user_data_cc : 1;                /* MPEG User Data is used
+  bool       user_data_cc : 1;                /* MPEG User Data is used
                                               for Closed Caption */
-  uint8_t use_lid2 : 1;                    /* If == 1 and the PSD is
+  bool       use_lid2 : 1;                    /* If == 1 and the PSD is
                                               interpreted and the next
                                               disc has the same album
                                               id then start the next
                                               disc at List ID #2,
                                               otherwise List ID #1 */ 
-  uint8_t use_track3 : 1;                  /* If == 1 and the PSD is
+  bool       use_track3 : 1;                  /* If == 1 and the PSD is
                                               not interpreted  and
                                               next disc has same album
                                               id, then start next disc
                                               with track 3, otherwise
                                               start with track 2 */ 
-  uint8_t pbc_x : 1;                       /* extended PBC available */
+  bool       pbc_x : 1;                       /* extended PBC available */
 #else
-  uint8_t pbc_x : 1;
-  uint8_t use_track3 : 1;
-  uint8_t use_lid2 : 1;
-  uint8_t user_data_cc : 1;
-  uint8_t special_info : 1;
-  uint8_t restriction : 2;
-  uint8_t reserved1 : 1;
+  bool       pbc_x : 1;
+  bool       use_track3 : 1;
+  bool       use_lid2 : 1;
+  bool       user_data_cc : 1;
+  bool       special_info : 1;
+  bitfield_t restriction : 2;
+  bool       reserved1 : 1;
 #endif
 } GNUC_PACKED InfoStatusFlags;
 
@@ -140,7 +142,7 @@ enum {
 typedef struct 
 {
 #if defined(BITFIELD_LSBF)
-  uint8_t audio_type : 2;                /* Audio characteristics:
+  bitfield_t audio_type : 2;                /* Audio characteristics:
                                             0x0 - No MPEG audio stream
                                             0x1 - One MPEG1 or MPEG2 audio
                                                   stream without extension
@@ -148,7 +150,7 @@ typedef struct
                                                   streams without extension
                                             0x3 - One MPEG2 multi-channel 
                                                     audio stream w/ extension*/
-  uint8_t video_type : 3;                /* Video characteristics:
+  bitfield_t video_type : 3;                /* Video characteristics:
                                             0x0 - No MPEG video data
                                             0x1 - NTSC still picture
                                             0x2 - Reserved (NTSC hires?)
@@ -157,20 +159,20 @@ typedef struct
                                             0x5 - PAL still picture
                                             0x6 - Reserved (PAL hires?)
                                             0x7 - PAL motion picture */
-  uint8_t item_cont : 1;                 /* Indicates segment is continuation
+  bool       item_cont : 1;                 /* Indicates segment is continuation
                                             0x0 - 1st or only segment of item
                                             0x1 - 2nd or later
                                                   segment of item */
-  uint8_t ogt : 2;                       /* 0x0 - no OGT substream 
+  bitfield_t ogt : 2;                       /* 0x0 - no OGT substream 
                                             0x1 - sub-stream 0 available
                                             0x2 - sub-stream 0 & 1 available
                                             0x3 - all OGT sub-substreams 
                                                   available */
 #else
-  uint8_t ogt : 2;
-  uint8_t item_cont : 1;
-  uint8_t video_type : 3;
-  uint8_t audio_type : 2;
+  bitfield_t ogt : 2;
+  bool       item_cont : 1;
+  bitfield_t video_type : 3;
+  bitfield_t audio_type : 2;
 #endif
 } GNUC_PACKED InfoSpiContents;
 
@@ -279,13 +281,13 @@ typedef struct {
 
 typedef struct {
 #if defined(BITFIELD_LSBF)
-  uint8_t SelectionAreaFlag : 1;
-  uint8_t CommandListFlag : 1;
-  uint8_t reserved : 6;
+  bool       SelectionAreaFlag : 1;
+  bool       CommandListFlag : 1;
+  bitfield_t reserved : 6;
 #else
-  uint8_t reserved : 6;
-  uint8_t CommandListFlag : 1;
-  uint8_t SelectionAreaFlag : 1;
+  bitfield_t reserved : 6;
+  bool       CommandListFlag : 1;
+  bool       SelectionAreaFlag : 1;
 #endif  
 } GNUC_PACKED PsdSelectionListFlags;
 
@@ -368,28 +370,28 @@ typedef struct {
 
 typedef struct {
 #if defined(BITFIELD_LSBF)
-  uint8_t audio : 2;                      /* Audio Content
+  bitfield_t audio : 2;                      /* Audio Content
                                              0x00 : No MPEG audio stream
                                              0x01 : One MPEG{1|2} audio stream
                                              0x02 : Two MPEG{1|2} streams
                                              0x03 : One MPEG2 multi-channel
                                                     audio stream with
                                                     extension */
-  uint8_t video : 3;                      /* Video Content
+  bitfield_t video : 3;                      /* Video Content
                                              0x00 : No MPEG video
                                              0x03 : NTSC video
                                              0x07 : PAL video */
-  uint8_t reserved1 : 1;                  /* Reserved, must be zero */
-  uint8_t ogt : 2;                        /* 0x0 - no OGT substream 
+  bool       reserved1 : 1;                  /* Reserved, must be zero */
+  bitfield_t ogt : 2;                        /* 0x0 - no OGT substream 
                                              0x1 - sub-stream 0 available
                                              0x2 - sub-stream 0 & 1 available
                                              0x3 - all OGT sub-substreams 
                                                    available */
 #else
-  uint8_t ogt : 2;
-  uint8_t reserved1 : 1;
-  uint8_t video : 3;
-  uint8_t audio : 2;
+  bitfield_t ogt : 2;
+  bool       reserved1 : 1;
+  bitfield_t video : 3;
+  bitfield_t audio : 2;
 #endif
 } GNUC_PACKED SVDTrackContent;
 
@@ -415,10 +417,11 @@ typedef struct {
 
 typedef struct {
   /* TracksSVD tracks_svd; */
-  SVDTrackContent contents[EMPTY_ARRAY_SIZE]; /* indicates track contents */
+  SVDTrackContent contents[1]; /* should be [], but C99 doesn't allow it
+                                  indicates track contents */
 } GNUC_PACKED TracksSVD2;
 
-#define TracksSVD2_SIZEOF 0
+#define TracksSVD2_SIZEOF SVDTrackContent_SIZEOF
 
 /* VCD30 tracks svd */
 
@@ -523,12 +526,12 @@ typedef struct {
 
 typedef struct {
   /* ScandataDat head; */
-  uint16_t spi_indexes[EMPTY_ARRAY_SIZE]; /* Indexes into
-                                             the following scandata
-                                             table */
+  uint16_t spi_indexes[1]; /* should be [], but C doesn't allow that;
+                              Indexes into the following scandata
+                              table */
 } GNUC_PACKED ScandataDat2;
 
-#define ScandataDat2_SIZEOF 0
+#define ScandataDat2_SIZEOF sizeof(uint16_t)
 
 typedef struct {
   /* ScandataDat2 head; */
@@ -548,10 +551,12 @@ typedef struct {
 
 typedef struct {
   /* ScandataDat3 head; */
-  msf_t scandata_table[EMPTY_ARRAY_SIZE];
+  msf_t scandata_table[1]; /* should be [] but C99 doesn't allow that */
 } GNUC_PACKED ScandataDat4;
 
-#define ScandataDat4_SIZEOF 0
+#define ScandataDat4_SIZEOF msf_t_SIZEOF
+
+PRAGMA_END_PACKED
 
 #endif /* __VCD_FILES_PRIVATE_H__ */
 
