@@ -1,7 +1,7 @@
 /*
-    $Id: vcd_image_nrg.c,v 1.11 2003/01/08 17:34:06 hvr Exp $
+    $Id: vcd_image_nrg.c,v 1.15 2003/02/24 16:47:56 rocky Exp $
 
-    Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
+    Copyright (C) 2001,2003 Herbert Valerio Riedel <hvr@gnu.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,6 +17,10 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
+
+/*! This code implements low-level access functions for Nero's native
+   CD-image format residing inside a disk file (*.nrg).
+*/
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -35,7 +39,7 @@
 #include <libvcd/vcd_assert.h>
 #include <string.h>
 
-static const char _rcsid[] = "$Id: vcd_image_nrg.c,v 1.11 2003/01/08 17:34:06 hvr Exp $";
+static const char _rcsid[] = "$Id: vcd_image_nrg.c,v 1.15 2003/02/24 16:47:56 rocky Exp $";
 
 /* structures used */
 
@@ -43,12 +47,13 @@ static const char _rcsid[] = "$Id: vcd_image_nrg.c,v 1.11 2003/01/08 17:34:06 hv
    least structure were set big endian, so at reverse
    engineering wasn't such a big headache... */
 
+PRAGMA_BEGIN_PACKED
 typedef struct {
   uint32_t start      GNUC_PACKED;
   uint32_t length     GNUC_PACKED;
   uint32_t type       GNUC_PACKED; /* only 0x3 seen so far... 
                                       -> MIXED_MODE2 2336 blocksize */
-  uint32_t start_lsn  GNUC_PACKED; /* cummulated w/o pre-gap! */
+  uint32_t start_lsn  GNUC_PACKED; /* does not include any pre-gaps! */
   uint32_t _unknown   GNUC_PACKED; /* wtf is this for? -- always zero... */
 } _etnf_array_t;
 
@@ -75,9 +80,13 @@ typedef struct {
   char data[EMPTY_ARRAY_SIZE]    GNUC_PACKED;
 } _chunk_t;
 
+PRAGMA_END_PACKED
+
 /* to be converted into BE */
 #define CUEX_ID  0x43554558
+#define CUES_ID  0x43554553
 #define DAOX_ID  0x44414f58
+#define DAOI_ID  0x44414f49
 #define END1_ID  0x454e4421
 #define ETN2_ID  0x45544e32
 #define ETNF_ID  0x45544e46
@@ -87,11 +96,16 @@ typedef struct {
 
 /* reader */
 
+#define DEFAULT_VCD_DEVICE "image.nrg"
+#define VCD_LEADOUT_TRACK  0xaa
+
 typedef struct {
   VcdDataSource *nrg_src;
+  char *nrg_name;
 
+  VcdList *mapping; /* List of track information */
   uint32_t size;
-  VcdList *mapping;
+  bool init;
 } _img_nrg_src_t;
 
 typedef struct {
@@ -111,6 +125,9 @@ _source_free (void *user_data)
   free (_obj);
 }
 
+/* Updates internal track TOC, so we can later 
+   simulate ioctl(CDROMREADTOCENTRY).
+ */
 static void
 _register_mapping (_img_nrg_src_t *_obj, 
 		  uint32_t start_lsn, uint32_t length_lsn,
@@ -131,8 +148,13 @@ _register_mapping (_img_nrg_src_t *_obj,
   /* vcd_debug ("map: %d +%d -> %ld", start_lsn, length_lsn, img_offset); */
 }
 
+
+/* 
+   Disk and track information for a Nero file are located at the end
+   of the file. This routine extracts that information.
+ */
 static int
-_parse_footer (_img_nrg_src_t *_obj)
+_parse_nero_footer (_img_nrg_src_t *_obj)
 {
   long size, footer_start;
   char *footer_buf = NULL;
@@ -143,6 +165,7 @@ _parse_footer (_img_nrg_src_t *_obj)
   size = vcd_data_source_stat (_obj->nrg_src);
 
   {
+PRAGMA_BEGIN_PACKED
     union {
       struct {
 	uint32_t __x          GNUC_PACKED;
@@ -154,6 +177,7 @@ _parse_footer (_img_nrg_src_t *_obj)
 	uint64_t footer_ofs   GNUC_PACKED;
       } v55;
     } buf;
+PRAGMA_END_PACKED
 
     vcd_assert (sizeof (buf) == 12);
 
@@ -351,14 +375,40 @@ _parse_footer (_img_nrg_src_t *_obj)
   return 0;
 }
 
-  static int
+/*!
+  Initialize image structures.
+ */
+static void
+_vcd_source_init (_img_nrg_src_t *_obj)
+{
+  if (_obj->init)
+    return;
+
+  if (!(_obj->nrg_src = vcd_data_source_new_stdio (_obj->nrg_name)))
+    vcd_error ("init failed");
+
+  _parse_nero_footer (_obj);
+  _obj->init = true;
+
+}
+
+static uint32_t 
+_vcd_stat_size (void *user_data)
+{
+  _img_nrg_src_t *_obj = user_data;
+
+  _vcd_source_init (_obj);
+  return _obj->size;
+}
+
+static int
 _read_mode2_sector (void *user_data, void *data, uint32_t lsn, bool form2)
 {
   _img_nrg_src_t *_obj = user_data;
   char buf[M2RAW_SECTOR_SIZE] = { 0, };
   VcdListNode *node;
 
-  _parse_footer (_obj);
+  _vcd_source_init (_obj);
 
   if (lsn >= _obj->size)
     {
@@ -394,31 +444,65 @@ _read_mode2_sector (void *user_data, void *data, uint32_t lsn, bool form2)
   return 0;
 }
 
-static uint32_t 
-_stat_size (void *user_data)
+/*!
+  Set the device to use in I/O operations.
+*/
+static int
+_source_set_arg (void *user_data, const char key[], const char value[])
 {
   _img_nrg_src_t *_obj = user_data;
 
-  _parse_footer (_obj);
-  return _obj->size;
+  if (!strcmp (key, "nrg"))
+    {
+      free (_obj->nrg_name);
+
+      if (!value)
+	return -2;
+
+      _obj->nrg_name = strdup (value);
+    }
+  else
+    return -1;
+
+  return 0;
+}
+
+/*!
+  Eject media -- there's nothing to do here. We always return 2.
+  also free obj.
+ */
+static int 
+_vcd_eject_media (void *user_data) {
+  /* Sort of a stub here. Perhaps log a message? */
+  return 2;
+}
+
+/*!
+  Return a string containing the default VCD device if none is specified.
+ */
+static char *
+_vcd_get_default_device()
+{
+  return strdup(DEFAULT_VCD_DEVICE);
 }
 
 VcdImageSource *
-vcd_image_source_new_nrg (VcdDataSource *nrg_source)
+vcd_image_source_new_nrg (void)
 {
   _img_nrg_src_t *_data;
 
   vcd_image_source_funcs _funcs = {
+    .eject_media       = _vcd_eject_media,
+    .free              = _source_free,
+    .get_default_device= _vcd_get_default_device,
     .read_mode2_sector = _read_mode2_sector,
-    .stat_size         = _stat_size,
-    .free              = _source_free
+    .set_arg           = _source_set_arg,
+    .stat_size         = _vcd_stat_size,
   };
 
-  if (!nrg_source)
-    return NULL;
-
   _data = _vcd_malloc (sizeof (_img_nrg_src_t));
-  _data->nrg_src = nrg_source;
+  _data->init              = false;
+  _data->nrg_name          = _vcd_get_default_device();
 
   return vcd_image_source_new (_data, &_funcs);
 }
