@@ -1,5 +1,5 @@
 /*
-    $Id: vcd_iso9660.h,v 1.7 2001/08/29 13:21:50 hvr Exp $
+    $Id: vcd_iso9660.h,v 1.8 2002/12/25 14:53:17 hvr Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -54,7 +54,7 @@ _vcd_iso_pathname_isofy (const char pathname[], uint16_t version);
 
 void
 set_iso_pvd (void *pd, const char volume_id[], const char application_id[],
-             const char publisher_id[],
+             const char publisher_id[], const char preparer_id[],
              uint32_t iso_size, const void *root_dir, 
              uint32_t path_table_l_extent, uint32_t path_table_m_extent,
              uint32_t path_table_size);
