@@ -1,5 +1,5 @@
 /*
-    $Id: vcd_logging.h,v 1.2 2001/05/22 17:08:24 hvr Exp $
+    $Id: vcd_logging.h,v 1.3 2002/07/23 17:47:37 hvr Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -23,6 +23,10 @@
 
 #include "vcd_types.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 typedef enum {
   LOG_DEBUG = 1,
   LOG_INFO,
@@ -51,6 +55,10 @@ vcd_warn (const char format[], ...) GNUC_PRINTF(1,2);
 void
 vcd_error (const char format[], ...) GNUC_PRINTF(1,2);
 
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
 #endif /* __VCD_LOGGING_H__ */
 
 
