@@ -1,5 +1,5 @@
 /*
-  $Id: vcd_mpeg.c,v 1.52 2003/01/19 15:09:59 hvr Exp $
+  $Id: vcd_mpeg.c,v 1.55 2003/03/16 10:52:50 hvr Exp $
 
   Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -28,7 +28,7 @@
 #include <libvcd/vcd_bitvec.h>
 #include <libvcd/vcd_util.h>
 
-static const char _rcsid[] = "$Id: vcd_mpeg.c,v 1.52 2003/01/19 15:09:59 hvr Exp $";
+static const char _rcsid[] = "$Id: vcd_mpeg.c,v 1.55 2003/03/16 10:52:50 hvr Exp $";
 
 #define MPEG_START_CODE_PATTERN  ((uint32_t) 0x00000100)
 #define MPEG_START_CODE_MASK     ((uint32_t) 0xffffff00)
@@ -320,11 +320,13 @@ _parse_user_data (uint8_t streamid, const void *buf, unsigned len,
                   VcdMpegStreamCtx *state)
 {
   unsigned pos = 0;
+  PRAGMA_BEGIN_PACKED
   struct {
     uint8_t tag;
     uint8_t len;
     uint8_t data[EMPTY_ARRAY_SIZE];
   } GNUC_PACKED const *udg = buf;
+  PRAGMA_END_PACKED
 
   if (udg->tag == 0x00) /* if first tag's already 0x00 */
     {
@@ -395,7 +397,7 @@ _analyze_pes_header (const uint8_t *buf, int len,
   bool _has_pts = false;
   bool _has_dts = false;
   int64_t pts = 0;
-  mpeg_vers_t pes_mpeg_ver = 0;
+  mpeg_vers_t pes_mpeg_ver = MPEG_VERS_INVALID;
 
   int pos;
 
@@ -505,8 +507,14 @@ _analyze_pes_header (const uint8_t *buf, int len,
           pos2 += 8;
           break;
 
+        case 0xf: /* %1111 - actually a syntax error! */
+          vcd_assert (vcd_bitvec_peek_bits (buf, pos2, 8) == 0xff);
+          vcd_warn ("Unexpected stuffing byte noticed in ISO11172 PES header!");
+          pos2 += 8;
+          break; 
+
         default:
-          vcd_error ("error in mpeg1 stream");
+          vcd_error ("Error in ISO11172 PES header");
           break;
         }
 
@@ -1152,7 +1160,7 @@ vcd_mpeg_packet_get_type (const struct vcd_mpeg_packet_info *_info)
   else if (_info->system_header || _info->padding)
     return PKT_TYPE_EMPTY;
 
-  return 0;
+  return PKT_TYPE_INVALID;
 }
 
 
