@@ -1,5 +1,5 @@
 /*
-    $Id: vcd_mpeg_stream.c,v 1.21 2001/08/28 14:00:28 hvr Exp $
+    $Id: vcd_mpeg_stream.c,v 1.22 2002/12/29 16:45:00 hvr Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -35,7 +35,7 @@
 #include <libvcd/vcd_mpeg.h>
 #include <libvcd/vcd_logging.h>
 
-static const char _rcsid[] = "$Id: vcd_mpeg_stream.c,v 1.21 2001/08/28 14:00:28 hvr Exp $";
+static const char _rcsid[] = "$Id: vcd_mpeg_stream.c,v 1.22 2002/12/29 16:45:00 hvr Exp $";
 
 struct _VcdMpegSource
 {
@@ -303,7 +303,7 @@ _approx_pts (VcdList *aps_list, uint32_t packet_no)
   return retval;
 }
 
-void static
+static void 
 _set_scan_msf (msf_t *_msf, long lsn)
 {
   if (lsn == -1)
@@ -317,7 +317,7 @@ _set_scan_msf (msf_t *_msf, long lsn)
   _msf->f |= 0x80;
 }
 
-void static
+static void 
 _fix_scan_info (struct vcd_mpeg_scan_data_t *scan_data_ptr,
                 unsigned packet_no, double pts, VcdList *aps_list)
 {
