@@ -1,5 +1,5 @@
 /*
-    $Id: vcd_pbc.c,v 1.26 2002/12/25 15:31:32 hvr Exp $
+    $Id: vcd_pbc.c,v 1.27 2003/02/19 16:38:33 hvr Exp $
 
     Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -34,7 +34,7 @@
 #include <libvcd/vcd_files_private.h>
 #include <libvcd/vcd_bytesex.h>
 
-static const char _rcsid[] = "$Id: vcd_pbc.c,v 1.26 2002/12/25 15:31:32 hvr Exp $";
+static const char _rcsid[] = "$Id: vcd_pbc.c,v 1.27 2003/02/19 16:38:33 hvr Exp $";
 
 static uint8_t
 _wtime (int seconds)
@@ -264,16 +264,16 @@ _vcd_pbc_node_length (const VcdObj *obj, const pbc_t *_pbc, bool extended)
 
     case PBC_PLAYLIST:
       n = _vcd_list_length (_pbc->item_id_list);
-      retval = offsetof (PsdPlayListDescriptor, itemid[n]);
+      retval = __vcd_offsetof (PsdPlayListDescriptor, itemid[n]);
       break;
 
     case PBC_SELECTION:
       n = _vcd_list_length (_pbc->select_id_list);
 
-      retval = offsetof (PsdSelectionListDescriptor, ofs[n]);
+      retval = __vcd_offsetof (PsdSelectionListDescriptor, ofs[n]);
 
       if (extended || _vcd_obj_has_cap_p (obj, _CAP_4C_SVCD))
-	retval += offsetof (PsdSelectionListDescriptorExtended, area[n]);
+	retval += __vcd_offsetof (PsdSelectionListDescriptorExtended, area[n]);
       break;
       
     case PBC_END:
