@@ -1,5 +1,5 @@
 /*
-    $Id: vcdinfo.c,v 1.16 2003/02/10 12:57:58 rocky Exp $
+    $Id: vcdinfo.c,v 1.29 2003/03/16 10:52:50 hvr Exp $
 
     Copyright (C) 2002,2003 Rocky Bernstein <rocky@panix.com>
 
@@ -53,6 +53,7 @@
 #include <libvcd/vcd_salloc.h>
 #include <libvcd/vcd_stream_stdio.h>
 #include <libvcd/vcd_image_bincue.h>
+#include <libvcd/vcd_image_nrg.h>
 #include <libvcd/vcd_image_cd.h>
 #include <libvcd/vcd_image_fs.h>
 #include <libvcd/vcd_xa.h>
@@ -60,8 +61,9 @@
 
 /* Eventually move above libvcd includes but having vcdinfo including. */
 #include "vcdinfo.h"
+#include "vcdinf.h"
 
-static const char _rcsid[] = "$Id: vcdinfo.c,v 1.16 2003/02/10 12:57:58 rocky Exp $";
+static const char _rcsid[] = "$Id: vcdinfo.c,v 1.29 2003/03/16 10:52:50 hvr Exp $";
 
 #define MIN_ENCODED_TRACK_NUM 100
 #define MIN_ENCODED_SEGMENT_NUM 1000
@@ -371,143 +373,6 @@ _getbuf (void)
   return _buf[_num];
 }
 
-/* Comparison routine used in sorting. */
-static int
-_offset_t_cmp (vcdinfo_offset_t *a, vcdinfo_offset_t *b)
-{
-  if (a->offset > b->offset)
-    return 1;
-
-  if (a->offset < b->offset)
-    return -1;
-  
-  return 0;
-}
-
-static void
-_visit_pbc (vcdinfo_obj_t *obj, unsigned int lid, unsigned int offset, 
-                   bool in_lot, bool ext)
-{
-  VcdListNode *node;
-  vcdinfo_offset_t *ofs;
-  unsigned psd_size = ext ? obj->psd_x_size : vcdinfo_get_psd_size (obj);
-  const uint8_t *psd = ext ? obj->psd_x : obj->psd;
-  unsigned _rofs = offset * obj->info.offset_mult;
-  VcdList *offset_list;
-
-  vcd_assert (psd_size % 8 == 0);
-
-  if (offset == PSD_OFS_DISABLED
-      || offset == PSD_OFS_MULTI_DEF 
-      || offset == PSD_OFS_MULTI_DEF_NO_NUM)
-    return;
-
-  if (_rofs >= psd_size)
-    {
-      vcd_warn ("psd offset out of range (%d >= %d)", _rofs, psd_size);
-      return;
-    }
-
-  vcd_assert (_rofs < psd_size);
-
-  if (!obj->offset_list)
-    obj->offset_list = _vcd_list_new ();
-
-  if (!obj->offset_x_list)
-    obj->offset_x_list = _vcd_list_new ();
-
-  offset_list = ext ? obj->offset_x_list : obj->offset_list;
-
-  _VCD_LIST_FOREACH (node, offset_list)
-    {
-      ofs = _vcd_list_node_data (node);
-
-      if (offset == ofs->offset)
-        {
-          if (in_lot)
-            ofs->in_lot = true;
-
-          if (lid) 
-            /* Our caller thinks she knows what our LID is.
-               This should help out getting the LID for end descriptors
-               if not other things as well.
-             */
-            ofs->lid = lid;
-
-          ofs->ext = ext;
-
-          return; /* already been there... */
-        }
-    }
-
-  ofs = _vcd_malloc (sizeof (vcdinfo_offset_t));
-
-  ofs->offset = offset;
-  ofs->lid    = lid;
-  ofs->in_lot = in_lot;
-  ofs->ext    = ext;
-
-  switch (psd[_rofs])
-    {
-    case PSD_TYPE_PLAY_LIST:
-      _vcd_list_append (offset_list, ofs);
-      {
-        const PsdPlayListDescriptor *d = (const void *) (psd + _rofs);
-        const lid_t lid = vcdinfo_get_lid_from_pld(d);
-
-        if (!ofs->lid)
-          ofs->lid = lid;
-        else 
-          if (ofs->lid != lid)
-            vcd_warn ("LOT entry assigned LID %d, but descriptor has LID %d",
-                      ofs->lid, lid);
-
-        _visit_pbc (obj, 0, uint16_from_be (d->prev_ofs), false, ext);
-        _visit_pbc (obj, 0, uint16_from_be (d->next_ofs), false, ext);
-        _visit_pbc (obj, 0, uint16_from_be (d->return_ofs), false, ext);
-      }
-      break;
-
-    case PSD_TYPE_EXT_SELECTION_LIST:
-    case PSD_TYPE_SELECTION_LIST:
-      _vcd_list_append (offset_list, ofs);
-      {
-        const PsdSelectionListDescriptor *d =
-          (const void *) (psd + _rofs);
-
-        int idx;
-
-        if (!ofs->lid)
-          ofs->lid = uint16_from_be (d->lid) & 0x7fff;
-        else 
-          if (ofs->lid != (uint16_from_be (d->lid) & 0x7fff))
-            vcd_warn ("LOT entry assigned LID %d, but descriptor has LID %d",
-                      ofs->lid, uint16_from_be (d->lid) & 0x7fff);
-
-        _visit_pbc (obj, 0, uint16_from_be (d->prev_ofs), false, ext);
-        _visit_pbc (obj, 0, uint16_from_be (d->next_ofs), false, ext);
-        _visit_pbc (obj, 0, uint16_from_be (d->return_ofs), false, ext);
-        _visit_pbc (obj, 0, uint16_from_be (d->default_ofs), false, ext);
-        _visit_pbc (obj, 0, uint16_from_be (d->timeout_ofs), false, ext);
-
-        for (idx = 0; idx < d->nos; idx++)
-          _visit_pbc (obj, 0, uint16_from_be (d->ofs[idx]), false, ext);
-        
-      }
-      break;
-
-    case PSD_TYPE_END_LIST:
-      _vcd_list_append (offset_list, ofs);
-      break;
-
-    default:
-      vcd_warn ("corrupt PSD???????");
-      free (ofs);
-      return;
-      break;
-    }
-}
-
 /*!
   \brief Classify itemid_num into the kind of item it is: track #, entry #, 
   segment #. 
@@ -515,7 +380,8 @@ _visit_pbc (vcdinfo_obj_t *obj, unsigned int lid, unsigned int offset,
   entry number. 
 */
 void
-vcdinfo_classify_itemid (uint16_t itemid_num, vcdinfo_itemid_t *itemid)
+vcdinfo_classify_itemid (uint16_t itemid_num, 
+                         /*out*/ vcdinfo_itemid_t *itemid)
 {
 
   itemid->num = itemid_num;
@@ -577,8 +443,8 @@ vcdinfo_pin2str (uint16_t itemid_num)
 const char *
 vcdinfo_get_album_id(const vcdinfo_obj_t *obj)
 {
-  if ( NULL == obj || NULL == &obj->info ) return (NULL);
-  return(obj->info.album_desc);
+  if ( NULL == obj ) return (NULL);
+  return vcdinf_get_album_id(&obj->info);
 }
 
 /*!
@@ -588,8 +454,8 @@ vcdinfo_get_album_id(const vcdinfo_obj_t *obj)
 const char *
 vcdinfo_get_application_id(const vcdinfo_obj_t *obj)
 {
-  if ( NULL == obj || NULL == &obj->pvd ) return (NULL);
-  return(vcdinfo_strip_trail(obj->pvd.application_id, MAX_APPLICATION_ID));
+  if ( NULL == obj ) return (NULL);
+  return(vcdinf_get_application_id(&obj->pvd));
 }
 
 /*!
@@ -599,16 +465,7 @@ vcdinfo_get_application_id(const vcdinfo_obj_t *obj)
 int
 vcdinfo_get_autowait_time (const PsdPlayListDescriptor *d) 
 {
-  return vcdinfo_calc_psd_wait_time (d->atime);
-}
-
-/*!
-   Return the base selection number.
-*/
-unsigned int
-vcdinfo_get_bsn(const PsdSelectionListDescriptor *psd)
-{
-  return(psd->bsn);
+  return vcdinf_get_wait_time (d->atime);
 }
 
 /**
@@ -674,14 +531,20 @@ uint32_t
 vcdinfo_get_entry_sect_count (const vcdinfo_obj_t *obj, unsigned int entry_num)
 {
   const EntriesVcd *entries = &obj->entries;
-  const unsigned int entry_count = uint16_from_be (entries->entry_count);
+  const unsigned int entry_count = vcdinf_get_num_entries(entries);
   if (entry_num > entry_count) 
     return 0;
   else {
     const lba_t this_lba = vcdinfo_get_entry_lba(obj, entry_num);
     lba_t next_lba;
     if (entry_num < entry_count-1) {
-       next_lba = vcdinfo_get_entry_lba(obj, entry_num+1);
+      track_t track=vcdinfo_get_track(obj, entry_num);
+      track_t next_track=vcdinfo_get_track(obj, entry_num+1);
+      next_lba = vcdinfo_get_entry_lba(obj, entry_num+1);
+      /* If we've changed tracks, don't include pregap sector between 
+         tracks.
+       */
+      if (track != next_track) next_lba -= PREGAP_SECTORS;
     } else {
       /* entry_num == entry_count -1. Or the last entry.
          This is really really ugly. There's probably a better
@@ -693,7 +556,7 @@ vcdinfo_get_entry_sect_count (const vcdinfo_obj_t *obj, unsigned int entry_num)
          *without* a pregap to the leadout track, we try not to use
          that if we can get the entry from the ISO 9660 filesystem.
       */
-      unsigned int track = vcdinfo_get_track(obj, entry_num);
+      track_t track = vcdinfo_get_track(obj, entry_num);
       if (track != VCDINFO_INVALID_TRACK) {
         vcd_image_stat_t statbuf;
         const lsn_t lsn = vcdinfo_lba2lsn(vcdinfo_get_track_lba(obj, track));
@@ -703,7 +566,8 @@ vcdinfo_get_entry_sect_count (const vcdinfo_obj_t *obj, unsigned int entry_num)
           const lsn_t next_lsn=lsn + statbuf.secsize;
           next_lba = vcdinfo_lsn2lba(next_lsn);
         } else {
-          /* Failed on ISO 9660 filesystem. Use LEADOUT track.  */
+          /* Failed on ISO 9660 filesystem. Use next track or
+             LEADOUT track.  */
           next_lba = vcdinfo_get_track_lba(obj, track+1);
         }
         if (next_lba == VCDINFO_NULL_LBA)
@@ -722,20 +586,17 @@ vcdinfo_get_entry_sect_count (const vcdinfo_obj_t *obj, unsigned int entry_num)
   The first entry number is 0.
 */
 const msf_t *
-vcdinfo_get_entry_msf(const vcdinfo_obj_t *obj, const unsigned int entry_num)
+vcdinfo_get_entry_msf(const vcdinfo_obj_t *obj, unsigned int entry_num)
 {
   const EntriesVcd *entries = &obj->entries;
-  const unsigned int entry_count = uint16_from_be (entries->entry_count);
-  return entry_num < entry_count ?
-    &(entries->entry[entry_num].msf)
-    : NULL;
+  return vcdinf_get_entry_msf(entries, entry_num);
 }
 
 /*!  Return the starting LBA (logical block address) for sequence
   entry_num in obj.  VCDINFO_NULL_LBA is returned if there is no entry.
 */
 lba_t
-vcdinfo_get_entry_lba(const vcdinfo_obj_t *obj, const unsigned int entry_num)
+vcdinfo_get_entry_lba(const vcdinfo_obj_t *obj, unsigned int entry_num)
 {
   const msf_t *msf = vcdinfo_get_entry_msf(obj, entry_num);
   return (msf != NULL) ? msf_to_lba(msf) : VCDINFO_NULL_LBA;
@@ -759,29 +620,7 @@ vcdinfo_get_format_version (vcdinfo_obj_t *obj)
 const char *
 vcdinfo_get_format_version_str (const vcdinfo_obj_t *obj) 
 {
-  switch (obj->vcd_type)
-    {
-    case VCD_TYPE_VCD:
-      return ("VCD 1.0");
-      break;
-    case VCD_TYPE_VCD11:
-      return ("VCD 1.1");
-      break;
-    case VCD_TYPE_VCD2:
-      return ("VCD 2.0");
-      break;
-    case VCD_TYPE_SVCD:
-      return ("SVCD");
-      break;
-    case VCD_TYPE_HQVCD:
-      return ("HQVCD");
-      break;
-    case VCD_TYPE_INVALID:
-      return ("INVALID");
-      break;
-    default:
-      return ( "????");
-    }
+  return vcdinf_get_format_version_str(obj->vcd_type);
 }
 
 /*!
@@ -799,7 +638,7 @@ vcdinfo_get_itemid_from_lid(const vcdinfo_obj_t *obj, lid_t lid)
   case PSD_TYPE_SELECTION_LIST:
   case PSD_TYPE_EXT_SELECTION_LIST:
     if (pxd.psd == NULL) return VCDINFO_REJECTED_MASK;
-    return vcdinfo_get_itemid_from_psd(pxd.psd);
+    return vcdinf_get_itemid_from_psd(pxd.psd);
     break;
   case PSD_TYPE_PLAY_LIST:
     /* FIXME: There is an array of items */
@@ -812,60 +651,6 @@ vcdinfo_get_itemid_from_lid(const vcdinfo_obj_t *obj, lid_t lid)
 
 }
 
-/*!
-  Get the item id for a given PSD selector descriptor. 
-  VCDINFO_REJECTED_MASK is returned on error or if d is NULL. 
-*/
-uint16_t
-vcdinfo_get_itemid_from_psd(const PsdSelectionListDescriptor *psd)
-{
-  return (psd != NULL) ? uint16_from_be(psd->itemid) : VCDINFO_REJECTED_MASK;
-}
-
-/* Get the LID from a given PSD selector descriptor. 
-   VCDINFO_REJECTED_MASK is returned d on error or d is NULL. 
-*/
-lid_t
-vcdinfo_get_lid_from_pld(const PsdPlayListDescriptor *d)
-{
-  return (d != NULL) 
-    ? uint16_from_be (d->lid) & VCDINFO_LID_MASK
-    : VCDINFO_REJECTED_MASK;
-}
-
-/*!
-  Get the LID from a given PSD play-list descriptor. 
-  VCDINFO_REJECTED_MASK is returned d on error or d is NULL. 
-*/
-lid_t
-vcdinfo_get_lid_from_psd(const PsdSelectionListDescriptor *d)
-{
-  return (d != NULL) 
-    ? uint16_from_be (d->lid) & VCDINFO_LID_MASK
-    : VCDINFO_REJECTED_MASK;
-}
-
-/*!
-  Get the LID rejected status for a given PSD selector descriptor. 
-  true is also returned d is NULL. 
-*/
-bool
-vcdinfo_get_lid_rejected_from_psd(const PsdSelectionListDescriptor *d)
-{
-  return (d != NULL) 
-    ? vcdinfo_is_rejected(uint16_from_be(d->lid)) 
-    : true;
-}
-
-/*!
-  Return number of times PLD loops.
-*/
-uint16_t
-vcdinfo_get_loop_count (const PsdSelectionListDescriptor *d) 
-{
-  return 0x7f & d->loop;
-}
-
 /*!
   Return number of LIDs. 
 */
@@ -873,33 +658,8 @@ lid_t
 vcdinfo_get_num_LIDs (const vcdinfo_obj_t *obj) 
 {
   /* Should probably use _vcd_pbc_max_lid instead? */
-  return uint16_from_be (obj->info.lot_entries);
-}
-
-/**
- \fn vcdinfo_get_next_from_pld(const PsdPlayListDescriptor *pld); 
- \brief  Get next offset for a given PSD selector descriptor.  
- \return  VCDINFO_INVALID_OFFSET is returned on error or if pld has no "next"
- entry or pld is NULL. Otherwise the LID offset is returned.
- */
-lid_t
-vcdinfo_get_next_from_pld(const PsdPlayListDescriptor *pld)
-{
-  return (pld != NULL) 
-    ? uint16_from_be (pld->next_ofs) : VCDINFO_INVALID_OFFSET;
-}
-
-/**
- * \fn vcdinfo_get_next_from_psd(const PsdSelectionListDescriptor *psd);
- * \brief Get next offset for a given PSD selector descriptor. 
- * \return VCDINFO_INVALID_OFFSET is returned on error or if psd is
- * NULL. Otherwise the LID offset is returned.
- */
-lid_t
-vcdinfo_get_next_from_psd(const PsdSelectionListDescriptor *psd)
-{
-  return (psd != NULL) 
-    ? uint16_from_be (psd->next_ofs) : VCDINFO_INVALID_OFFSET;
+  if (NULL==obj) return 0;
+  return vcdinf_get_num_LIDs(&obj->info);
 }
 
 /*!
@@ -909,62 +669,18 @@ unsigned int
 vcdinfo_get_num_entries(const vcdinfo_obj_t *obj)
 {
   const EntriesVcd *entries = &obj->entries;
-  return (uint16_from_be (entries->entry_count));
+  return vcdinf_get_num_entries(entries);
 }
 
 /*!
-  Return the number of segments in the VCD. 
+  Return the number of segments in the VCD. Return 0 if there is some
+  problem.
 */
 unsigned int
 vcdinfo_get_num_segments(const vcdinfo_obj_t *obj)
 {
-  const InfoVcd *info = &obj->info;
-  return (uint16_from_be (info->item_count));
-}
-
-/*!
-  \fn vcdinfo_get_offset_from_lid(const vcdinfo_obj_t *obj, unsigned int entry_num);
-  \brief Get offset entry_num for a given LID. 
-  \return VCDINFO_INVALID_OFFSET is returned if obj on error or obj
-  is NULL. Otherwise the LID offset is returned.
-*/
-uint16_t
-vcdinfo_get_offset_from_lid(const vcdinfo_obj_t *obj, lid_t lid,
-			    unsigned int entry_num) 
-{
-  PsdListDescriptor pxd;
-
-  if (obj == NULL) return VCDINFO_INVALID_OFFSET;
-  vcdinfo_get_pxd_from_lid(obj, &pxd, lid);
-
-  switch (pxd.descriptor_type) {
-  case PSD_TYPE_SELECTION_LIST:
-  case PSD_TYPE_EXT_SELECTION_LIST:
-    if (pxd.psd == NULL) return VCDINFO_INVALID_OFFSET;
-    return vcdinfo_get_offset_from_psd(pxd.psd, entry_num-1);
-    break;
-  case PSD_TYPE_PLAY_LIST:
-    /* FIXME: There is an array of items */
-  case PSD_TYPE_END_LIST:
-  case PSD_TYPE_COMMAND_LIST:
-    return VCDINFO_INVALID_OFFSET;
-  }
-  return VCDINFO_INVALID_OFFSET;
-
-}
-
-/**
- * \fn vcdinfo_get_offset_from_psd(const PsdSelectionListDescriptor *d, unsigned int entry_num);
- * \brief Get offset entry_num for a given PSD selector descriptor. 
- * \return VCDINFO_INVALID_OFFSET is returned if d on error or d is
- * NULL. Otherwise the LID offset is returned.
- */
-uint16_t
-vcdinfo_get_offset_from_psd(const PsdSelectionListDescriptor *d, 
-			    unsigned int entry_num) 
-{
-  return (d != NULL || entry_num < d->nos) 
-    ? uint16_from_be (d->ofs[entry_num]) : VCDINFO_INVALID_OFFSET;
+  if (NULL==obj) return 0;
+  return vcdinf_get_num_segments(&obj->info);
 }
 
 /*! 
@@ -1006,16 +722,6 @@ vcdinfo_get_offset_t (const vcdinfo_obj_t *obj, unsigned int offset)
   return _vcdinfo_get_offset_t (obj, offset, false);
 }
 
-/*!
-  Get autowait time value for PsdPlayListDescriptor *d.
-  Time is in seconds.
-*/
-uint16_t
-vcdinfo_get_play_time (const PsdPlayListDescriptor *d) 
-{
-  return uint16_from_be (d->ptime);
-}
-
 /*!
    Return a string containing the VCD publisher id with trailing
    blanks removed, or NULL if there is some problem in getting this.
@@ -1023,43 +729,18 @@ vcdinfo_get_play_time (const PsdPlayListDescriptor *d)
 const char *
 vcdinfo_get_preparer_id(const vcdinfo_obj_t *obj)
 {
-  if ( NULL == obj || NULL == &obj->pvd ) return (NULL);
-  return(vcdinfo_strip_trail(obj->pvd.preparer_id, MAX_PREPARER_ID));
-}
-
-/**
- \fn vcdinfo_get_prev_from_pld(const PsdPlayListDescriptor *pld);
- \brief Get prev offset for a given PSD selector descriptor. 
- \return  VCDINFO_INVALID_OFFSET is returned on error or if pld has no "prev"
- entry or pld is NULL. Otherwise the LID offset is returned.
- */
-lid_t
-vcdinfo_get_prev_from_pld(const PsdPlayListDescriptor *pld)
-{
-  return (pld != NULL) ? 
-    uint16_from_be (pld->prev_ofs) : VCDINFO_INVALID_OFFSET;
-}
-
-/**
- \fn vcdinfo_get_prev_from_psd(const PsdSelectionListDescriptor *psd);
- \brief Get prev offset for a given PSD selector descriptor. 
- \return  VCDINFO_INVALID_OFFSET is returned on error or if psd has no "prev"
- entry or psd is NULL. Otherwise the LID offset is returned.
- */
-lid_t
-vcdinfo_get_prev_from_psd(const PsdSelectionListDescriptor *psd)
-{
-  return (psd != NULL) ? 
-    uint16_from_be (psd->prev_ofs) : VCDINFO_INVALID_OFFSET;
+  if ( NULL == obj ) return (NULL);
+  return vcdinf_get_preparer_id(&obj->pvd);
 }
 
 /*!
-  Return number of bytes in PSD. 
+  Return number of bytes in PSD. Return 0 if there's an error.
 */
 uint32_t
 vcdinfo_get_psd_size (const vcdinfo_obj_t *obj)
 {
-  return uint32_from_be (obj->info.psd_size);
+  if ( NULL == obj ) return 0;
+  return vcdinf_get_psd_size(&obj->info);
 }
 
 /*!
@@ -1069,8 +750,8 @@ vcdinfo_get_psd_size (const vcdinfo_obj_t *obj)
 const char *
 vcdinfo_get_publisher_id(const vcdinfo_obj_t *obj)
 {
-  if ( NULL == obj || NULL == &obj->pvd ) return (NULL);
-  return(vcdinfo_strip_trail(obj->pvd.publisher_id, 128));
+  if ( NULL == obj ) return (NULL);
+  return vcdinf_get_publisher_id(&obj->pvd);
 }
 
 /*!
@@ -1100,7 +781,7 @@ _vcdinfo_get_pxd_from_lid(const vcdinfo_obj_t *obj, PsdListDescriptor *pxd,
         case PSD_TYPE_PLAY_LIST:
           {
             pxd->pld = (PsdPlayListDescriptor *) (psd + _rofs);
-            if (vcdinfo_get_lid_from_pld(pxd->pld) == lid) {
+            if (vcdinf_get_lid_from_pld(pxd->pld) == lid) {
               return true;
             }
             break;
@@ -1110,7 +791,7 @@ _vcdinfo_get_pxd_from_lid(const vcdinfo_obj_t *obj, PsdListDescriptor *pxd,
         case PSD_TYPE_SELECTION_LIST: 
           {
             pxd->psd = (PsdSelectionListDescriptor *) (psd + _rofs);
-            if (vcdinfo_get_lid_from_psd(pxd->psd) == lid) {
+            if (vcdinf_get_lid_from_psd(pxd->psd) == lid) {
               return true;
             }
             break;
@@ -1135,7 +816,7 @@ vcdinfo_get_pxd_from_lid(const vcdinfo_obj_t *obj, PsdListDescriptor *pxd,
 }
 
 /**
- \fn vcdinfo_get_return_from_psd(const PsdPlayListDescriptor *pld);
+ \fn vcdinfo_get_return(const vcdinfo_obj_t *obj);
  \brief Get return offset for a given PLD selector descriptor. 
  \return  VCDINFO_INVALID_OFFSET is returned on error or if pld has no 
  "return" entry or pld is NULL. Otherwise the LID offset is returned.
@@ -1168,32 +849,6 @@ vcdinfo_get_return(const vcdinfo_obj_t *obj, unsigned int lid)
   return VCDINFO_INVALID_OFFSET;
 }
 
-/**
- \fn vcdinfo_get_return_from_psd(const PsdPlayListDescriptor *pld);
- \brief Get return offset for a given PLD selector descriptor. 
- \return  VCDINFO_INVALID_OFFSET is returned on error or if pld has no 
- "return" entry or pld is NULL. Otherwise the LID offset is returned.
- */
-lid_t
-vcdinfo_get_return_from_pld(const PsdPlayListDescriptor *pld)
-{
-  return (pld != NULL) ? 
-    uint16_from_be (pld->return_ofs) : VCDINFO_INVALID_OFFSET;
-}
-
-/**
- * \fn vcdinfo_get_return_from_psd(const PsdSelectionListDescriptor *psd);
- * \brief Get return offset for a given PSD selector descriptor. 
- \return  VCDINFO_INVALID_OFFSET is returned on error or if psd has no 
- "return" entry or psd is NULL. Otherwise the LID offset is returned.
- */
-uint16_t
-vcdinfo_get_return_from_psd(const PsdSelectionListDescriptor *psd)
-{
-  return (psd != NULL) ? 
-    uint16_from_be (psd->return_ofs) : VCDINFO_INVALID_OFFSET;
-}
-
 /*!
    Return the audio type for a given segment. 
    VCDINFO_INVALID_AUDIO_TYPE is returned on error.
@@ -1294,7 +949,7 @@ const char *
 vcdinfo_get_system_id(const vcdinfo_obj_t *obj)
 {
   if ( NULL == obj || NULL == &obj->pvd ) return (NULL);
-  return(vcdinfo_strip_trail(obj->pvd.system_id, MAX_SYSTEM_ID));
+  return(vcdinf_get_system_id(&obj->pvd));
 }
 
 /*!
@@ -1314,7 +969,7 @@ vcdinfo_get_timeout_LID (const PsdSelectionListDescriptor *d)
 int
 vcdinfo_get_timeout_time (const PsdSelectionListDescriptor *d)
 {
-  return vcdinfo_calc_psd_wait_time (d->totime);
+  return vcdinf_get_wait_time (d->totime);
 }
 
 /*!
@@ -1322,15 +977,14 @@ vcdinfo_get_timeout_time (const PsdSelectionListDescriptor *d)
   at 1. Note this is one less than the track number reported in vcddump.
   (We don't count the header track?)
 */
-unsigned int
+track_t
 vcdinfo_get_track(const vcdinfo_obj_t *obj, const unsigned int entry_num)
 {
   const EntriesVcd *entries = &obj->entries;
-  const unsigned int entry_count = uint16_from_be (entries->entry_count);
+  const unsigned int entry_count = vcdinf_get_num_entries(entries);
   /* Note entry_num is 0 origin. */
   return entry_num < entry_count ?
-    from_bcd8 (entries->entry[entry_num].n)-1:
-    VCDINFO_INVALID_TRACK;
+    vcdinf_get_track(entries, entry_num)-1: VCDINFO_INVALID_TRACK;
 }
 
 /*!
@@ -1340,7 +994,7 @@ vcdinfo_get_track(const vcdinfo_obj_t *obj, const unsigned int entry_num)
    Note: track 1 is usually the first track.
 */
 unsigned int
-vcdinfo_get_track_audio_type(const vcdinfo_obj_t *obj, unsigned int track_num)
+vcdinfo_get_track_audio_type(const vcdinfo_obj_t *obj, track_t track_num)
 {
   TracksSVD *tracks;
   TracksSVD2 *tracks2;
@@ -1367,13 +1021,17 @@ vcdinfo_get_num_tracks(const vcdinfo_obj_t *obj)
 
 /*!  
   Return the starting LBA (logical block address) for track number
-  track_num in obj.  Tracks numbers start at 1.
+  track_num in obj.  
+
+  The IS0-9660 filesystem track has number 0. Tracks associated
+  with playable entries numbers start at 1.
+
   The "leadout" track is specified either by
   using track_num LEADOUT_TRACK or the total tracks+1.
   VCDINFO_NULL_LBA is returned on failure.
 */
 lba_t 
-vcdinfo_get_track_lba(const vcdinfo_obj_t *obj, unsigned int track_num)
+vcdinfo_get_track_lba(const vcdinfo_obj_t *obj, track_t track_num)
 {
   if (NULL == obj || NULL == obj->img)
     return VCDINFO_NULL_LBA;
@@ -1396,7 +1054,7 @@ vcdinfo_get_track_lba(const vcdinfo_obj_t *obj, unsigned int track_num)
   O is returned on success; 1 is returned on error.
 */
 int
-vcdinfo_get_track_msf(const vcdinfo_obj_t *obj, unsigned int track_num,
+vcdinfo_get_track_msf(const vcdinfo_obj_t *obj, track_t track_num,
                       uint8_t *min, uint8_t *sec, uint8_t *frame)
 {
   if (NULL == obj || NULL == obj->img)
@@ -1424,11 +1082,17 @@ vcdinfo_get_track_msf(const vcdinfo_obj_t *obj, unsigned int track_num,
 }
 
 /*!
-  Return the size in sectors for track n. The first track is 1.
+  Return the size in sectors for track n. 
+
+  The IS0-9660 filesystem track has number 0. Tracks associated
+  with playable entries numbers start at 1.
+
+  FIXME: Whether we count the track pregap sectors is a bit haphazard.
+  We should add a parameter to indicate whether this is wanted or not.
+
 */
 unsigned int
-vcdinfo_get_track_sect_count(const vcdinfo_obj_t *obj, 
-                             const unsigned int track_num)
+vcdinfo_get_track_sect_count(const vcdinfo_obj_t *obj, const track_t track_num)
 {
   if (NULL == obj || VCDINFO_INVALID_TRACK == track_num) 
     return 0;
@@ -1439,7 +1103,7 @@ vcdinfo_get_track_sect_count(const vcdinfo_obj_t *obj,
     const lsn_t lsn = vcdinfo_lba2lsn(lba);
     
     /* Try to get the sector count from the ISO 9660 filesystem */
-    if (_vcdinfo_find_fs_lsn(obj, &statbuf, lsn)) {
+    if (obj->has_xa && _vcdinfo_find_fs_lsn(obj, &statbuf, lsn)) {
       return statbuf.secsize;
     } else {
       const lba_t next_lba=vcdinfo_get_track_lba(obj, track_num+1);
@@ -1451,7 +1115,13 @@ vcdinfo_get_track_sect_count(const vcdinfo_obj_t *obj,
 }
 
 /*!
-  Return the size in bytes for track n. The first track is 1.
+  Return size in bytes for track number for entry n in obj.
+
+  The IS0-9660 filesystem track has number 0. Tracks associated
+  with playable entries numbers start at 1.
+
+  FIXME: Whether we count the track pregap sectors is a bit haphazard.
+  We should add a parameter to indicate whether this is wanted or not.
 */
 unsigned int
 vcdinfo_get_track_size(const vcdinfo_obj_t *obj, const unsigned int track_num)
@@ -1499,8 +1169,8 @@ vcdinfo_get_video_type(const vcdinfo_obj_t *obj, unsigned int seg_num)
 unsigned int
 vcdinfo_get_volume_count(const vcdinfo_obj_t *obj)
 {
-  if ( NULL == obj || NULL == &obj->info ) return 0;
-  return(uint16_from_be( obj->info.vol_count));
+  if ( NULL == obj ) return 0;
+  return vcdinf_get_volume_count(&obj->info);
 }
 
 /*!
@@ -1511,7 +1181,7 @@ const char *
 vcdinfo_get_volume_id(const vcdinfo_obj_t *obj)
 {
   if ( NULL == obj || NULL == &obj->pvd ) return (NULL);
-  return(vcdinfo_strip_trail(obj->pvd.volume_id, 32));
+  return(vcdinf_get_volume_id(&obj->pvd));
 }
 
 /*!
@@ -1533,7 +1203,7 @@ vcdinfo_get_volumeset_id(const vcdinfo_obj_t *obj)
 unsigned int
 vcdinfo_get_volume_num(const vcdinfo_obj_t *obj)
 {
-  if ( NULL == obj || NULL == &obj->info ) return 0;
+  if ( NULL == obj ) return 0;
   return(uint16_from_be( obj->info.vol_id));
 }
 
@@ -1544,7 +1214,7 @@ vcdinfo_get_volume_num(const vcdinfo_obj_t *obj)
 int
 vcdinfo_get_wait_time (const PsdPlayListDescriptor *d) 
 {
-  return vcdinfo_calc_psd_wait_time (d->wtime);
+  return vcdinf_get_wait_time (d->wtime);
 }
 
 /*!
@@ -1631,7 +1301,7 @@ void
 vcdinfo_inc_msf (uint8_t *min, uint8_t *sec, int8_t *frame)
 {
   (*frame)++;
-  if (*frame>=75) {
+  if (*frame>=CD_FRAMES_PER_SECOND) {
     *frame = 0;
     (*sec)++;
     if (*sec>=60) {
@@ -1649,7 +1319,7 @@ vcdinfo_inc_msf (uint8_t *min, uint8_t *sec, int8_t *frame)
 lba_t
 vcdinfo_msf2lba (uint8_t min, uint8_t sec, int8_t frame)
 {
-  return 75*(60*min + sec) + frame;
+  return CD_FRAMES_PER_SECOND*(60*min + sec) + frame;
 }
 
 /*!
@@ -1747,54 +1417,30 @@ vcdinfo_read_psd (vcdinfo_obj_t *obj)
   return true;
 }
 
-/*
-  This fills in the offset table with LIDs.  Due to
-  "rejected" LOT entries, some of these might not have gotten filled
-  in before.
-
-  I believe a requirement for this to work is that the offset_list be
-  sorted beforehand: to get the LIDs we are assuming first in the
-  offset list is LID1, second in the offset LID2 etc.
- */
-static void
-vcdinfo_update_offset_list(const vcdinfo_obj_t *obj, bool ext)
-{
-  if (NULL==obj) return;
-  {
-    VcdListNode *node;
-    unsigned int lid=1;
-    VcdList *offset_list = ext ? obj->offset_x_list : obj->offset_list;
-    
-    _VCD_LIST_FOREACH (node, offset_list)
-      {
-        vcdinfo_offset_t *ofs = _vcd_list_node_data (node);
-        ofs->lid = lid++;
-      }
-  }
-}
-
 void
-vcdinfo_visit_lot (vcdinfo_obj_t *obj, bool ext)
+vcdinfo_visit_lot (vcdinfo_obj_t *obj, bool extended)
 {
-  const LotVcd *lot = ext ? obj->lot_x : obj->lot;
-  unsigned n;
+  struct _vcdinf_pbc_ctx pbc_ctx;
 
-  if (!ext && !vcdinfo_get_psd_size (obj))
-    return;
+  pbc_ctx.psd_size      = vcdinfo_get_psd_size (obj);
+  pbc_ctx.psd_x_size    = obj->psd_x_size;
+  pbc_ctx.offset_mult   = 8;
+  pbc_ctx.maximum_lid   = vcdinfo_get_num_LIDs(obj);
+  pbc_ctx.offset_x_list = NULL;
+  pbc_ctx.offset_list   = NULL;
+  pbc_ctx.psd           = obj->psd;
+  pbc_ctx.psd_x         = obj->psd_x;
+  pbc_ctx.lot           = obj->lot;
+  pbc_ctx.lot_x         = obj->lot_x;
+  pbc_ctx.extended      = extended;
 
-  if (ext && !obj->psd_x_size)
-    return;
-
-  for (n = 0; n < LOT_VCD_OFFSETS; n++)
-    _visit_pbc (obj, n + 1, uint16_from_be (lot->offset[n]), true, ext);
-
-  _vcd_list_sort (ext ? obj->offset_x_list : obj->offset_list, 
-                  (_vcd_list_cmp_func) _offset_t_cmp);
-
-  /* Now really complete the offset table with LIDs.  This routine
-     might obviate the need for _visit_pbc() or some of it which is
-     more complex. */
-  vcdinfo_update_offset_list(obj, ext);
+  vcdinf_visit_lot(&pbc_ctx);
+  if (NULL != obj->offset_x_list) 
+    _vcd_list_free(obj->offset_x_list, true);
+  obj->offset_x_list = pbc_ctx.offset_x_list;
+  if (NULL != obj->offset_list) 
+    _vcd_list_free(obj->offset_list, true);
+  obj->offset_list   = pbc_ctx.offset_list;
 }
 
 /*!
@@ -1833,18 +1479,6 @@ vcdinfo_is_rejected(uint16_t offset)
   return (offset & VCDINFO_REJECTED_MASK) != 0;
 }
 
-int
-vcdinfo_calc_psd_wait_time (uint16_t wtime)
-{
-  /* Note: this doesn't agree exactly with _wtime */
-  if (wtime < 61)
-    return wtime;
-  else if (wtime < 255)
-    return (wtime - 60) * 10 + 60;
-  else
-    return -1;
-}
-
 const char *
 vcdinfo_area_str (const struct psd_area_t *_area)
 {
@@ -1883,109 +1517,35 @@ vcdinfo_init(vcdinfo_obj_t *obj)
 }
 
 /*!
-   Set up vcdinfo structure "obj" for reading and writing from a
-   particular medium. This should be done before after initialization
-   but before any routines that need to retrieve or write data.
+   Set up vcdinfo structure "obj" for reading from a particular
+   medium. This should be done before after initialization but before
+   any routines that need to retrieve data.
  
    source_name is the device or file to use for inspection, and
    source_type indicates if this is a device or a file.
+   access_mode gives special access options for reading.
     
    VCDINFO_OPEN_VCD is returned if everything went okay; 
    VCDINFO_OPEN_ERROR if there was an error and VCDINFO_OPEN_OTHER if the
    medium is something other than a VCD.
  */
 vcdinfo_open_return_t
-vcdinfo_open(vcdinfo_obj_t *obj, char source_name[], 
-             vcdinfo_source_t source_type)
+vcdinfo_open(vcdinfo_obj_t *obj, char *source_name[], 
+             vcdinfo_source_t source_type, const char access_mode[])
 {
-
   VcdImageSource *img;
-  char is_sector_2352 = true;
-  bool null_source = NULL == source_name;
+  bool null_source = NULL == *source_name;
 
-  if (VCDINFO_SOURCE_AUTO == source_type) {
-    if (null_source) 
-      source_type = VCDINFO_SOURCE_DEVICE;
-    else {
-      struct stat buf;
-      if (0 != stat(source_name, &buf)) {
-        vcd_error ("Can't stat file %s:", strerror(errno));
-        return VCDINFO_OPEN_ERROR;
-      }
-      if (S_ISBLK(buf.st_mode) || S_ISCHR(buf.st_mode)) {
-        source_type = VCDINFO_SOURCE_DEVICE;
-      } else if (S_ISREG(buf.st_mode)) {
-        /* FIXME: check to see if is a text file. If so, then 
-           set VCDINFO_SOURCE_CUE. */
-        int i=strlen(source_name)-strlen("bin");
-        if (i > 0
-            && (source_name[i] =='c' || source_name[i+1] =='C')
-            && (source_name[i+1] =='u' || source_name[i+1] =='U')
-            && (source_name[i+2] =='e' || source_name[i+2] =='E') ) 
-          source_type = VCDINFO_SOURCE_CUE;
-        else
-          source_type = VCDINFO_SOURCE_BIN;
-      } else {
-        vcd_error ("Source file `%s' should either be a block device "
-                   "or a regular file", source_name);
-        return VCDINFO_OPEN_ERROR;
-      }
-    }
-  }
-  
-  switch (source_type) {
-  case VCDINFO_SOURCE_DEVICE:
-    img = vcd_image_source_new_cd ();
-    if (null_source) {
-      source_name=vcd_image_source_get_default_device(img);
-    }
-    vcd_image_source_set_arg (img, "device", source_name);
-    break;
-  case VCDINFO_SOURCE_SECTOR_2336:
-    is_sector_2352 = false;
-    /* Fall through to next case */
-  case VCDINFO_SOURCE_BIN:
-    img = vcd_image_source_new_bincue ();
-    if (null_source) {
-      source_name=vcd_image_source_get_default_device(img);
-    }
-    vcd_image_source_set_arg (img, "bin", source_name);
-    vcd_image_source_set_arg (img, "sector", is_sector_2352 ? "2352": "2336");
-    break;
-  case VCDINFO_SOURCE_CUE:
-    img = vcd_image_source_new_bincue ();
-    if (null_source) {
-      source_name=vcd_image_source_get_default_device(img);
-    } else {
-      int i=strlen(source_name)-strlen("cue");
-      if (i>0) {
-        char *bin_name=strdup(source_name);
-        if (bin_name[i]=='c') {
-          bin_name[i++]='b'; bin_name[i++]='i'; bin_name[i++]='n';
-          vcd_image_source_set_arg (img, "bin", bin_name);
-        } else if (bin_name[i]=='C') {
-          bin_name[i++]='B'; bin_name[i++]='I'; bin_name[i++]='N';
-          vcd_image_source_set_arg (img, "bin", bin_name);
-        }
-        vcd_image_source_set_arg (img, "cue", source_name);
-      } else {
-        return VCDINFO_OPEN_ERROR;
-      }
-      
-    }
-    vcd_image_source_set_arg (img, "sector", is_sector_2352 ? "2352": "2336");
-    break;
-  default: 
+  if (!vcdinf_open(&img, source_name, source_type, access_mode)) 
     return VCDINFO_OPEN_ERROR;
-  }
-  
+    
   memset (obj, 0, sizeof (vcdinfo_obj_t));
   obj->img = img;  /* Note we do this after the above wipeout! */
   obj->segment_list = _vcd_list_new ();
   
   if (vcd_image_source_read_mode2_sector (obj->img, &(obj->pvd), 
                                           ISO_PVD_SECTOR, false))
-    return(false);
+    return VCDINFO_OPEN_ERROR;
 
   /* Determine if VCD has XA attributes. */
   {
@@ -1999,7 +1559,7 @@ vcdinfo_open(vcdinfo_obj_t *obj, char source_name[],
     
   if (vcd_image_source_read_mode2_sector (obj->img, &(obj->info), 
                                           INFO_VCD_SECTOR, false))
-    return(false);
+    return VCDINFO_OPEN_ERROR;
 
   if (vcdinfo_get_format_version (obj) == VCD_TYPE_INVALID)
     return VCDINFO_OPEN_OTHER;
@@ -2009,13 +1569,13 @@ vcdinfo_open(vcdinfo_obj_t *obj, char source_name[],
     return VCDINFO_OPEN_OTHER;
   
   {
-    size_t len = strlen(source_name)+1;
+    size_t len = strlen(*source_name)+1;
     obj->source_name = (char *) malloc(len * sizeof(char));
-    strncpy(obj->source_name, source_name, len);
+    strncpy(obj->source_name, *source_name, len);
   }
 
   if (null_source) {
-    free(source_name);
+    free(*source_name);
   }
 
   if (obj->vcd_type == VCD_TYPE_SVCD || obj->vcd_type == VCD_TYPE_HQVCD) {
@@ -2034,13 +1594,13 @@ vcdinfo_open(vcdinfo_obj_t *obj, char source_name[],
         
         if (vcd_image_source_read_mode2_sector (obj->img, obj->tracks_buf, 
                                                 statbuf.lsn, false))
-          return(false);
+          return VCDINFO_OPEN_ERROR;
       }
   }
       
   _init_fs_segment (obj, "/", true);
 
-  return(true);
+  return VCDINFO_OPEN_VCD;
   
 }
 
@@ -2073,8 +1633,13 @@ vcdinfo_close(vcdinfo_obj_t *obj)
 {
   if (obj != NULL) {
     if (obj->segment_list != NULL) 
-      _vcd_list_free(obj->segment_list, 0);
+      _vcd_list_free(obj->segment_list, true);
+    if (obj->offset_list != NULL) 
+      _vcd_list_free(obj->offset_list, true);
+    if (obj->offset_x_list != NULL) 
+      _vcd_list_free(obj->offset_x_list, true);
     free(obj->lot);
+    free(obj->lot_x);
     free(obj->tracks_buf);
     free(obj->search_buf);
     free(obj->source_name);
