@@ -1,5 +1,5 @@
 /*
-    $Id: check_bitfield.c,v 1.1 2002/12/25 15:31:32 hvr Exp $
+    $Id: check_bitfield.c,v 1.2 2003/02/19 16:38:33 hvr Exp $
 
     Copyright (C) 2001 Herbert Valerio Riedel <hvr@gnu.org>
 
@@ -300,12 +300,12 @@ main (int argc, const char *argv[])
       const union {
 	InfoSpiContents bf;
 	uint8_t byte;
-      } u = { .bf = InfoSpiContents_table[i].bitfield };
+      } *u = (void *) &InfoSpiContents_table[i].bitfield;
 
-      if (u.byte != InfoSpiContents_table[i].intval)
+      if (u->byte != InfoSpiContents_table[i].intval)
 	{
 	  printf ("failed at index %d (%.2x != %.2x)\n", 
-		  i, u.byte, InfoSpiContents_table[i].intval);
+		  i, u->byte, InfoSpiContents_table[i].intval);
 	  return 1;
 	}
     }
